from ._core import (
    NODEFAULT,
    Meta,
    Struct,
    StructConfig,
    StructMeta,
    TarsDict,
    TraceNode,
    ValidationError,
    decode_trace,
    field,
    inspect,
    probe_struct,
)
from .api import decode, encode

__version__ = "0.5.1"

__all__ = [
    "NODEFAULT",
    "Meta",
    "Struct",
    "StructConfig",
    "StructMeta",
    "TarsDict",
    "TraceNode",
    "ValidationError",
    "decode",
    "decode_trace",
    "encode",
    "field",
    "inspect",
    "probe_struct",
]



"""测试 Schema/Meta/Inspect 相关行为."""

from typing import Annotated, Generic, Optional, TypeVar, cast

import pytest
from tarsio import inspect as tinspect
from tarsio._core import (
    NODEFAULT,
    Meta,
    Struct,
    TarsDict,
    ValidationError,
    encode_raw,
    field,
)


def test_classic_mode_allows_extra_non_meta_annotations() -> None:
    """经典模式允许额外的非 Meta 注解."""

    class User(Struct):
        name: Annotated[str, 2, "doc"]

    obj = User("a")
    assert obj.name == "a"


def test_mix_int_tag_and_meta_constraints_allowed() -> None:
    """整数 Tag 与 Meta 约束可组合使用."""

    class User(Struct):
        uid: Annotated[int, 1, Meta(gt=0)]

    data = encode_raw(TarsDict({1: 7}))
    obj = User.decode(data)
    assert obj.uid == 7


def test_meta_without_tag_is_allowed() -> None:
    """Meta 不再承载 tag，缺省由自动分配生效."""

    class User(Struct):
        name: Annotated[str, Meta(max_len=10)]

    info = tinspect.struct_info(User)
    assert info is not None
    assert info.fields[0].tag == 0


def test_annotated_without_explicit_tag_auto_assigns() -> None:
    """Annotated 字段缺省 tag 时应自动分配."""

    class User(Struct):
        uid: Annotated[int, "doc"]

    info = tinspect.struct_info(User)
    assert info is not None
    assert info.fields[0].tag == 0


def test_plain_annotations_auto_assign_tags() -> None:
    """全普通注解字段应按定义顺序自动分配 Tag."""

    class User(Struct):
        uid: int
        name: str

    info = tinspect.struct_info(User)
    assert info is not None
    assert [f.tag for f in info.fields] == [0, 1]
    assert [f.name for f in info.fields] == ["uid", "name"]


def test_plain_annotations_roundtrip_succeeds() -> None:
    """全普通注解字段应可正常编解码."""

    class User(Struct):
        uid: int
        name: str

    obj = User(uid=7, name="alice")
    restored = User.decode(obj.encode())
    assert restored.uid == 7
    assert restored.name == "alice"


def test_mix_plain_and_annotated_is_allowed() -> None:
    """普通注解与 Annotated 可混用并自动分配 tag."""

    class User(Struct):
        uid: int
        name: Annotated[str, Meta(min_len=1)]

    info = tinspect.struct_info(User)
    assert info is not None
    assert [f.tag for f in info.fields] == [0, 1]


def test_field_tag_supports_explicit_and_implicit_mix() -> None:
    """field(tag=...) 与自动分配可混合使用."""

    class User(Struct):
        a: int = field(tag=0)
        b: Annotated[int, Meta(gt=0)]
        c: str = field(tag=5)
        d: bytes

    info = tinspect.struct_info(User)
    assert info is not None
    assert [(f.name, f.tag) for f in info.fields] == [
        ("a", 0),
        ("b", 1),
        ("c", 5),
        ("d", 6),
    ]


def test_multiple_integer_tags_raises() -> None:
    """同一字段声明多个整数 Tag 时抛出 TypeError."""
    with pytest.raises(TypeError, match="Multiple integer tags"):

        class User(Struct):
            uid: Annotated[int, 1, 2]


def test_multiple_meta_objects_raises() -> None:
    """同一字段声明多个 Meta 时抛出 TypeError."""
    with pytest.raises(TypeError, match="Multiple Meta objects"):

        class User(Struct):
            uid: Annotated[int, Meta(gt=0), Meta(le=10)]


def test_duplicate_tag_raises() -> None:
    """两个字段重复使用同一 Tag 时抛出 TypeError."""
    with pytest.raises(TypeError, match="Duplicate tag"):

        class User(Struct):
            a: Annotated[int, 1]
            b: Annotated[int, 1]


def test_tag_upper_limit_raises_error() -> None:
    """Tag 超过上限 255 时抛出 ValueError / TypeError."""
    # 根据目前的具体实现可能抛出 TypeError (Schema构建时) 或者是 ValueError
    with pytest.raises((TypeError, ValueError)):

        class User(Struct):
            a: Annotated[int, 256]


def test_tag_negative_raises_error() -> None:
    """负数 Tag 时抛出 ValueError / TypeError."""
    with pytest.raises((TypeError, ValueError)):

        class User(Struct):
            a: Annotated[int, -1]


def test_meta_constraints_with_field_tag_is_allowed() -> None:
    """Meta 约束与 field(tag=...) 组合应允许正常解码."""

    class User(Struct):
        uid: Annotated[int, Meta(ge=0)] = field(tag=1)

    data = encode_raw(TarsDict({1: 123}))
    obj = User.decode(data)
    assert obj.uid == 123


def test_numeric_gt_validation_raises() -> None:
    """数值 gt 约束不满足时抛出 ValidationError."""

    class User(Struct):
        uid: Annotated[int, Meta(gt=0)] = field(tag=1)

    data = encode_raw(TarsDict({1: 0}))
    with pytest.raises(ValidationError, match="must be >"):
        User.decode(data)


@pytest.mark.parametrize("value", [0, 10], ids=["lower", "upper"])
def test_numeric_ge_le_validation_passes(value: int) -> None:
    """数值区间约束满足时正常解码."""

    class User(Struct):
        uid: Annotated[int, Meta(ge=0, le=10)] = field(tag=1)

    ok = User.decode(encode_raw(TarsDict({1: value})))
    assert ok.uid == value


def test_numeric_ge_validation_raises() -> None:
    """数值 ge 约束不满足时抛出 ValidationError."""

    class User(Struct):
        uid: Annotated[int, Meta(ge=1)] = field(tag=1)

    with pytest.raises(ValidationError, match="must be >="):
        User.decode(encode_raw(TarsDict({1: 0})))


def test_string_min_len_validation_raises() -> None:
    """字符串 min_len 约束不满足时抛出 ValidationError."""

    class User(Struct):
        name: Annotated[str, Meta(min_len=1, max_len=3)] = field(tag=1)

    with pytest.raises(ValidationError, match="length must be >="):
        User.decode(encode_raw(TarsDict({1: ""})))


def test_string_max_len_validation_raises() -> None:
    """字符串 max_len 约束不满足时抛出 ValidationError."""

    class User(Struct):
        name: Annotated[str, Meta(min_len=1, max_len=3)] = field(tag=1)

    with pytest.raises(ValidationError, match="length must be <="):
        User.decode(encode_raw(TarsDict({1: "abcd"})))


def test_string_pattern_validation_passes() -> None:
    """字符串满足 pattern 约束时正常解码."""

    class User(Struct):
        code: Annotated[str, Meta(pattern=r"^[A-Z]{2}\d{2}$")] = field(tag=1)

    ok = User.decode(encode_raw(TarsDict({1: "AA12"})))
    assert ok.code == "AA12"


def test_string_pattern_validation_raises() -> None:
    """字符串不满足 pattern 约束时抛出 ValidationError."""

    class User(Struct):
        code: Annotated[str, Meta(pattern=r"^[A-Z]{2}\d{2}$")] = field(tag=1)

    with pytest.raises(ValidationError, match="does not match pattern"):
        User.decode(encode_raw(TarsDict({1: "aa12"})))


def test_tuple_length_constraints_validation_raises() -> None:
    """Tuple 字段应应用 min_len/max_len 约束."""

    class User(Struct):
        data: Annotated[tuple[int, int], Meta(min_len=3)] = field(tag=1)

    with pytest.raises(ValidationError, match="length must be >="):
        User.decode(encode_raw(TarsDict({1: (1, 2)})))


def test_incompatible_constraints_type_raises() -> None:
    """约束与值类型不兼容时应抛出 ValidationError."""

    class User(Struct):
        uid: Annotated[int, Meta(min_len=1)] = field(tag=1)

    with pytest.raises(ValidationError, match="must have length"):
        User.decode(encode_raw(TarsDict({1: 10})))


def test_init_decode_constraint_error_type_consistent() -> None:
    """同一约束在构造与解码阶段应保持同类异常."""

    class User(Struct):
        uid: Annotated[int, Meta(gt=0)] = field(tag=1)

    with pytest.raises(ValidationError, match="must be >"):
        User(uid=0)

    with pytest.raises(ValidationError, match="must be >"):
        User.decode(encode_raw(TarsDict({1: 0})))


def test_type_info_primitives() -> None:
    """type_info 应能解析基础类型."""
    assert tinspect.type_info(int).kind == "int"
    assert tinspect.type_info(str).kind == "str"
    assert tinspect.type_info(float).kind == "float"
    assert tinspect.type_info(bool).kind == "bool"
    assert tinspect.type_info(bytes).kind == "bytes"


def test_type_info_containers() -> None:
    """type_info 应能解析容器类型."""
    assert tinspect.type_info(list[int]).kind == "list"
    assert tinspect.type_info(tuple[str]).kind == "tuple"
    assert tinspect.type_info(dict[str, int]).kind == "map"


def test_type_info_optional() -> None:
    """type_info 应能解析 Optional."""
    info = tinspect.type_info(Optional[int])  # noqa: UP045
    assert info.kind == "optional"


def test_type_info_unsupported_type_raises() -> None:
    """不支持类型应抛 TypeError."""
    with pytest.raises(TypeError, match="Unsupported Tars type"):
        tinspect.type_info(complex)  # type: ignore[arg-type]


def test_struct_info_fields_and_order() -> None:
    """struct_info 应按 Tag 顺序返回字段."""

    class Sample(Struct):
        b: Annotated[int, 1]
        a: Annotated[str, 0]

    info = tinspect.struct_info(Sample)
    assert info is not None
    assert [f.name for f in info.fields] == ["a", "b"]


def test_struct_info_generic_template_returns_fields() -> None:
    """未具体化泛型模板的 struct_info 应返回字段信息."""
    t_type = TypeVar("t_type")

    class Box(Struct, Generic[t_type]):
        value: Annotated[t_type, 0]

    info = tinspect.struct_info(Box)
    assert info is not None
    assert info.fields[0].name == "value"


def test_struct_info_generic_template_and_concrete_diff() -> None:
    """模板与具体化泛型的字段类型应反映差异."""
    t_type = TypeVar("t_type")

    class Box(Struct, Generic[t_type]):
        value: Annotated[t_type, 0]

    template_info = tinspect.struct_info(Box)
    assert template_info is not None
    assert template_info.fields[0].type.kind == "any"

    concrete_info = tinspect.struct_info(Box[int])
    assert concrete_info is not None
    assert concrete_info.fields[0].type.kind == "int"


def test_struct_info_meta_constraints() -> None:
    """Meta 约束应被解析到 constraints."""

    class Sample(Struct):
        a: Annotated[int, Meta(gt=0)] = field(tag=0)

    info = tinspect.struct_info(Sample)
    assert info is not None
    field_info = info.fields[0]
    int_type = cast(tinspect.IntType, field_info.type)
    assert int_type.gt == 0


def test_fieldinfo_alias_to_field() -> None:
    """FieldInfo 应作为 Field 的兼容别名可用."""

    class Sample(Struct):
        a: Annotated[int, 0]

    info = tinspect.struct_info(Sample)
    assert info is not None
    field = info.fields[0]
    assert isinstance(field, tinspect.Field)
    assert tinspect.FieldInfo is tinspect.Field


def test_struct_info_field_default_factory_visible() -> None:
    """struct_info 字段应暴露 default_factory 并与 default 区分."""

    class Sample(Struct):
        a: Annotated[list[int], 0] = field(default_factory=list)

    info = tinspect.struct_info(Sample)
    assert info is not None
    f = info.fields[0]
    assert f.has_default is True
    assert f.default is NODEFAULT
    assert f.default_factory is list


def test_struct_info_annotated_without_tag_auto_assigns() -> None:
    """Annotated 字段未显式指定 tag 时应自动分配."""

    class Sample(Struct):
        a: Annotated[int, "doc"]  # type: ignore

    info = tinspect.struct_info(Sample)
    assert info is not None
    assert info.fields[0].tag == 0


def test_struct_info_duplicate_tag_raises() -> None:
    """重复 Tag 应抛 TypeError."""
    with pytest.raises(TypeError, match="Duplicate tag"):

        class Sample(Struct):
            a: Annotated[int, 1]
            b: Annotated[int, 1]


def test_struct_info_mix_annotated_tag_and_field_tag_raises() -> None:
    """Annotated 整数 tag 与 field(tag=...) 同时设置应抛 TypeError."""
    with pytest.raises(
        TypeError,
        match="cannot mix Annotated integer tag with field\\(tag=\\.\\.\\.\\)",
    ):

        class Sample(Struct):
            a: Annotated[int, 1] = field(tag=1)



"""提供 Tarsio 测试的公共 Fixtures 和配置."""

from collections.abc import Generator
from io import BytesIO
from typing import Annotated, Optional

import pytest
from tarsio import Struct

# ==========================================
# 基础测试结构体
# ==========================================


class User(Struct):
    """用于测试的简单用户结构体."""

    id: Annotated[int, 0]
    name: Annotated[str, 1]


class OptionalUser(Struct):
    """带可选字段的用户结构体."""

    id: Annotated[int, 0]
    name: Annotated[str | None, 1] = None
    age: Annotated[int | None, 2] = None


class Node(Struct):
    """用于测试递归结构的链表节点."""

    val: Annotated[int, 0]
    next: Annotated[Optional["Node"], 1] = None


# ==========================================
# Fixtures
# ==========================================


@pytest.fixture
def sample_user() -> User:
    """提供一个预置的 User 实例.

    Returns:
        User: 初始化后的 User 对象，id=42, name="Alice"。
    """
    return User(42, "Alice")


@pytest.fixture
def sample_optional_user() -> OptionalUser:
    """提供一个带可选字段的 OptionalUser 实例.

    Returns:
        OptionalUser: 初始化后的 OptionalUser 对象，id=1, name="Bob", age=None。
    """
    return OptionalUser(1, "Bob", None)


@pytest.fixture
def sample_node() -> Node:
    """提供一个链表节点实例.

    Returns:
        Node: 初始化后的 Node 对象，val=100, next=Node(200, None)。
    """
    return Node(100, Node(200, None))


@pytest.fixture
def mock_stream() -> Generator[BytesIO, None, None]:
    """提供内存字节流环境.

    Yields:
        BytesIO: 一个已打开的内存字节流实例。
    """
    stream = BytesIO()
    yield stream
    stream.close()



"""Primitives benchmarks."""

import pytest
from tarsio._core import decode, encode

from .models import HighTagStruct, Primitives


@pytest.mark.benchmark(group="primitives")
def test_bench_encode_primitives(benchmark, primitives_obj):
    """测试基础类型编码性能."""
    benchmark(encode, primitives_obj)


@pytest.mark.benchmark(group="primitives")
def test_bench_decode_primitives(benchmark, primitives_bytes):
    """测试基础类型解码性能."""
    benchmark(decode, Primitives, primitives_bytes)


@pytest.mark.benchmark(group="primitives")
def test_bench_encode_primitives_zeros(benchmark, primitives_zeros):
    """测试 ZeroTag 优化路径编码性能."""
    benchmark(encode, primitives_zeros)


@pytest.mark.benchmark(group="high_tag")
def test_bench_encode_high_tag(benchmark, high_tag_obj):
    """测试高 Tag 字段编码性能."""
    benchmark(encode, high_tag_obj)


@pytest.mark.benchmark(group="high_tag")
def test_bench_decode_high_tag(benchmark, high_tag_bytes):
    """测试高 Tag 字段解码性能."""
    benchmark(decode, HighTagStruct, high_tag_bytes)



"""Error handling benchmarks."""

import pytest
from tarsio._core import decode

from .models import Primitives


@pytest.mark.benchmark(group="error_handling")
def test_bench_decode_error(benchmark, malformed_bytes):
    """测试畸形数据解码失败的开销 (Fail Fast)."""

    def _decode_safe():
        with pytest.raises(ValueError):  # noqa: PT011
            decode(Primitives, malformed_bytes)

    benchmark(_decode_safe)



"""Tars 协议基线测试.

验证编解码规则、边界情况与 Raw 模式行为。
此文件定义的 hex 字节流必须与 Rust 核心层实现完全匹配 (Big Endian, Tagged Lengths).
"""

import pytest
from tarsio._core import (
    TarsDict,
    decode,
    decode_raw,
    decode_trace,
    encode_raw,
    probe_struct,
)


@pytest.mark.parametrize(
    ("val", "expected_hex"),
    [
        (0, "1C"),  # Tag 1, ZeroTag(12) -> 1C
        (1, "1001"),  # Tag 1, Int1(0) -> 10, Val 01
        (-128, "1080"),  # Tag 1, Int1, Val 80 (i8)
        (127, "107F"),  # Tag 1, Int1, Val 7F
        (255, "1100FF"),  # Tag 1, Int2(1) -> 11, Val 00FF (Big Endian)
        (32767, "117FFF"),  # Tag 1, Int2, Val 7FFF
        (-32768, "118000"),  # Tag 1, Int2, Val 8000
        (65535, "120000FFFF"),  # Tag 1, Int4(2) -> 12, Val 0000FFFF
        (2147483647, "127FFFFFFF"),  # Tag 1, Int4, Val 7FFFFFFF
    ],
    ids=[
        "zero",
        "one",
        "int1_min",
        "int1_max",
        "int2_255",
        "int2_max",
        "int2_min",
        "int4_65535",
        "int4_max",
    ],
)
def test_encode_int_values(val: int, expected_hex: str) -> None:
    """测试不同范围整数的编码 (Raw Struct {1: val})."""
    data = encode_raw(TarsDict({1: val}))
    assert data.hex().upper() == expected_hex.replace(" ", "")


@pytest.mark.parametrize(
    ("val", "expected_hex"),
    [
        ("", "1600"),  # Tag 1, String1(6) -> 16, Len 0
        ("a", "160161"),  # Tag 1, String1, Len 1, 'a'
        ("hello", "160568656C6C6F"),
        ("你好", "1606E4BDA0E5A5BD"),
    ],
    ids=["empty_string", "single_char", "ascii_hello", "chinese_nihao"],
)
def test_encode_string_values(val: str, expected_hex: str) -> None:
    """测试字符串编码 (Raw Struct {1: val})."""
    data = encode_raw(TarsDict({1: val}))
    assert data.hex().upper() == expected_hex


def test_encode_decode_extra_long_string() -> None:
    """测试超长字符串的编解码 (String4 边界及以上)."""
    # 临界点: 255 字节 (String1 最大容量)
    s_255 = "a" * 255
    data_255 = encode_raw(TarsDict({1: s_255}))
    assert decode_raw(data_255)[1] == s_255

    # 临界点以上: 256 字节 (进入 String4)
    s_256 = "b" * 256
    data_256 = encode_raw(TarsDict({1: s_256}))
    assert decode_raw(data_256)[1] == s_256

    # 超大字符串 (1MB)
    s_1mb = "c" * (1024 * 1024)
    data_1mb = encode_raw(TarsDict({1: s_1mb}))
    assert decode_raw(data_1mb)[1] == s_1mb


def test_encode_two_fields_struct() -> None:
    """测试多字段结构体编码顺序与 Tag."""
    # {0: 1, 1: "a"}
    # Tag 0, Int1(0): 00 01
    # Tag 1, String1: 16 01 61
    data = encode_raw(TarsDict({0: 1, 1: "a"}))
    assert data.hex().upper() == "0001160161"


def test_encode_nested_struct() -> None:
    """测试嵌套结构体编码."""
    # {0: {1: 100}}
    # Tag 0, StructBegin(10): 0A
    #   Tag 1, Int1(0): 10 64 (100)
    # StructEnd(11): 0B
    inner = TarsDict({1: 100})
    data = encode_raw(TarsDict({0: inner}))
    assert data.hex().upper() == "0A10640B"


def test_encode_deeply_nested_struct() -> None:
    """测试深层嵌套."""
    # {0: {0: {0: 1}}}
    # 0A -> 0A -> 00 01 -> 0B -> 0B
    # Inner: Tag 0, Int1(0), Val 1 -> 00 01
    val = TarsDict({0: TarsDict({0: TarsDict({0: 1})})})
    data = encode_raw(val)
    assert data.hex().upper() == "0A0A00010B0B"


# --- Decode Tests ---


def test_decode_int_roundtrip() -> None:
    """测试整数解码回原值."""
    for val in [0, 1, -1, 127, -128, 255, 65535, 2147483647]:
        data = encode_raw(TarsDict({0: val}))
        decoded = decode_raw(data)
        assert decoded[0] == val


def test_decode_invalid_utf8_raises_value_error() -> None:
    """无效 UTF-8 字节串解码应抛出 ValueError."""
    # Tag 1, String1, len 1, 0xFF
    data = bytes.fromhex("1601FF")
    with pytest.raises(ValueError, match="Invalid UTF-8"):
        decode_raw(data)


def test_decode_two_fields_struct() -> None:
    """测试多字段解码."""
    # 00 01 (Tag 0: 1), 16 01 61 (Tag 1: "a")
    data = bytes.fromhex("0001160161")
    res = decode_raw(data)
    assert res[0] == 1
    assert res[1] == "a"


def test_decode_nested_struct() -> None:
    """测试嵌套结构解码."""
    # 0A 10 64 0B
    data = bytes.fromhex("0A10640B")
    res = decode_raw(data)
    assert isinstance(res[0], TarsDict)
    assert res[0][1] == 100


def test_encode_raw_allows_direct_struct_value() -> None:
    """Raw 模式应允许 TarsDict 直接字段值为 Struct."""
    from typing import Annotated

    from tarsio import Struct

    class Inner(Struct):
        val: Annotated[int, 0]

    data = encode_raw(TarsDict({0: Inner(7)}))
    decoded = decode_raw(data)
    assert isinstance(decoded[0], TarsDict)
    assert decoded[0][0] == 7


def test_decode_raw_map_struct_value_returns_tarsdict() -> None:
    """Raw 解码时 map 内 StructBegin 应返回 TarsDict."""
    from typing import Annotated

    from tarsio import Struct

    class Inner(Struct):
        val: Annotated[int, 0]

    data = encode_raw(TarsDict({0: {"k": Inner(7)}}))
    decoded = decode_raw(data)
    assert isinstance(decoded[0]["k"], TarsDict)
    assert decoded[0]["k"][0] == 7


def test_decode_raw_deep_nested_struct_returns_tarsdict() -> None:
    """Raw 解码时深层容器中的 StructBegin 应统一返回 TarsDict."""
    from typing import Annotated

    from tarsio import Struct

    class Inner(Struct):
        val: Annotated[int, 0]

    data = encode_raw(TarsDict({0: {"items": [Inner(3)]}}))
    decoded = decode_raw(data)
    assert isinstance(decoded[0]["items"][0], TarsDict)
    assert decoded[0]["items"][0][0] == 3


def test_decode_skips_unknown_tag() -> None:
    """测试跳过未知 Tag (Schema 模式)."""
    from typing import Annotated

    from tarsio import Struct

    class User(Struct):
        uid: Annotated[int, 0]

    # Data: Tag 0=1, Tag 1="a" -> 00 01 16 01 61
    data = bytes.fromhex("0001160161")
    user = decode(User, data)
    assert user.uid == 1


def test_decode_with_type_mismatch_raises_value_error() -> None:
    """类型不匹配抛出异常."""
    from typing import Annotated

    from tarsio import Struct

    class User(Struct):
        uid: Annotated[int, 0]

    # Data has tag 0 as String "a" -> 06 01 61
    data = bytes.fromhex("060161")
    with pytest.raises(ValueError, match="Failed to read int"):
        decode(User, data)


def test_encode_raw_map_complex() -> None:
    """测试 Raw 模式 Map 编码."""
    # {0: {"a": 1}} -> Tag 0 Map(8) -> 08
    # Size 1 (Int1 Tag 0) -> 00 01
    # Key "a" (Tag 0 String): 06 01 61
    # Val 1 (Tag 1 Int1): 10 01
    val = TarsDict({0: {"a": 1}})
    data = encode_raw(val)
    assert data.hex().upper() == "0800010601611001"


def test_encode_raw_list_complex() -> None:
    """测试 Raw 模式 List 编码."""
    # {0: [1]} -> Tag 0 List(9) -> 09
    # Size 1 (Int1 Tag 0) -> 00 01
    # Item 1 (Tag 0 Int1): 00 01  <-- List items usually use implicit tags or Tag 0?
    # Tars List: Head (Tag 0, Type 9), Length (Tag 0 Int), Items...
    # Items in list don't have tags in strict Tars?
    # However, any_codec.rs calls `serialize_any(writer, 0, &item...)`.
    # So items are encoded with Tag 0.
    val = TarsDict({0: [1]})
    data = encode_raw(val)
    assert data.hex().upper() == "0900010001"


def test_decode_raw_max_depth_exceeded() -> None:
    """测试 Raw 解码深度限制."""
    data = bytes.fromhex("0A" * 101 + "0B" * 101)
    with pytest.raises(ValueError, match="Recursion depth exceeded"):
        decode_raw(data)


def test_float_double() -> None:
    """测试浮点数编码."""
    d = TarsDict({0: 1.5})
    data = encode_raw(d)
    # Tag 0, Double(5) -> 05
    # 1.5 Double Big Endian: 3FF8000000000000
    assert data.hex().upper() == "053FF8000000000000"


def test_bytes_simplelist() -> None:
    """测试 bytes (SimpleList)."""
    val = b"\x01\x02"
    d = TarsDict({0: val})
    data = encode_raw(d)
    # Tag 0, SimpleList(13) -> 0D
    # Subtype 0 (Byte) -> 00
    # Len 2 (Int1 Tag 0) -> 00 02
    # Data -> 01 02
    assert data.hex().upper() == "0D0000020102"


def test_schema_wrap_simplelist_field_writes_simplelist_wire() -> None:
    """Schema 字段 wrap_simplelist=True 应写出 SimpleList 线协议."""
    from typing import Annotated

    from tarsio import Struct, field

    class Inner(Struct):
        val: Annotated[int, 0]

    class Outer(Struct):
        inner: Annotated[Inner, 0] = field(wrap_simplelist=True)

    wire = Outer(inner=Inner(9)).encode()
    assert wire[0] == 0x0D

    payload_len = wire[3]
    payload = wire[4 : 4 + payload_len]
    restored = Inner.decode(payload)
    assert restored.val == 9


def test_raw_accepts_bytearray_and_memoryview_as_simplelist() -> None:
    """Raw 编码应将 bytearray/memoryview 统一按 SimpleList(bytes) 编码."""
    for value in (bytearray(b"\x01\x02"), memoryview(b"\x01\x02")):
        data = encode_raw(TarsDict({0: value}))
        assert data.hex().upper() == "0D0000020102"


def test_raw_accepts_non_contiguous_memoryview_as_simplelist() -> None:
    """Raw 编码应支持非连续 memoryview 并自动拷贝."""
    view = memoryview(bytearray(b"abcdef"))[::2]
    data = encode_raw(TarsDict({0: view}))
    decoded = decode_raw(data)
    assert decoded[0] == b"ace"
    assert isinstance(decoded[0], bytes)


def test_simplelist_always_returns_bytes() -> None:
    """验证 SimpleList 始终返回 bytes."""
    # Construct manually: 0D 00 00 02 11 01
    # Tag 0 SimpleList, Byte type, Len 2, Data 11 01
    data = bytes.fromhex("0D0000021101")
    decoded = decode_raw(data)
    assert isinstance(decoded[0], bytes)
    assert decoded[0] == b"\x11\x01"


def test_decode_raw_accepts_buffer_protocol_input() -> None:
    """decode_raw 应接受 bytearray 和 memoryview 输入."""
    payload = bytes.fromhex("0D0000021101")

    decoded_ba = decode_raw(bytearray(payload))
    decoded_mv = decode_raw(memoryview(payload))

    assert decoded_ba[0] == b"\x11\x01"
    assert decoded_mv[0] == b"\x11\x01"


def test_raw_recursion_limit() -> None:
    """验证递归深度限制."""
    d = TarsDict({})
    curr = d
    for _ in range(101):
        curr[0] = TarsDict({})
        curr = curr[0]
    with pytest.raises(ValueError, match="Recursion depth exceeded"):
        encode_raw(d)


def test_decode_trace_depth_exceeded() -> None:
    """decode_trace 深度超过阈值应抛 ValueError."""
    data = bytes.fromhex("0A" * 101 + "0B" * 101)
    with pytest.raises(ValueError, match="Trace recursion depth exceeded"):
        decode_trace(data)


def test_probe_struct_valid() -> None:
    """测试 probe_struct 有效性."""
    # {0: 1, 1: "s"} -> 00 01 16 01 73
    data = bytes.fromhex("0001160173")
    assert probe_struct(data) == {0: 1, 1: "s"}

    # Invalid
    assert probe_struct(bytes.fromhex("0F")) is None
    # Truncated
    assert probe_struct(bytes.fromhex("0A11")) is None


def test_decode_schema_accepts_buffer_protocol_input() -> None:
    """Schema decode 应接受 bytearray 和 memoryview 输入."""
    from tarsio import Struct

    class User(Struct):
        uid: int

    data = encode_raw(TarsDict({0: 7}))
    u1 = decode(User, bytearray(data))
    u2 = decode(User, memoryview(data))

    assert u1.uid == 7
    assert u2.uid == 7



"""Tarsio Raw benchmark small data."""

import pytest
from tarsio._core import decode_raw, encode_raw


@pytest.mark.benchmark(group="raw_small")
def test_bench_raw_encode_small(benchmark, raw_data_small):
    """测试 Raw 模式编码简单字典."""
    benchmark(encode_raw, raw_data_small)


@pytest.mark.benchmark(group="raw_small")
def test_bench_raw_decode_small(benchmark, raw_bytes_small):
    """测试 Raw 模式解码简单字典."""
    benchmark(decode_raw, raw_bytes_small)



"""Tarsio 统一编解码 API.

本模块提供了核心的序列化与反序列化接口，封装了底层的 Rust 实现，
并提供了完整的类型提示与文档。
"""

from typing import Any, TypeVar, overload

from ._core import Struct, TarsDict
from ._core import (
    decode as _core_decode,
)
from ._core import (
    decode_raw as _core_decode_raw,
)
from ._core import (
    encode as _core_encode,
)
from ._core import (
    encode_raw as _core_encode_raw,
)

_StructT = TypeVar("_StructT")
_BytesLike = bytes | bytearray | memoryview

__all__ = [
    "decode",
    "encode",
]


def encode(obj: Any) -> bytes:
    """将对象序列化为 Tars 二进制格式.

    该函数会自动根据输入对象的类型选择合适的编码模式：
    1. **Schema 模式**：如果对象是 `Struct`、`dataclass`、`NamedTuple` 或 `TypedDict` 实例，
       将按照其定义的 Schema 进行编码。
    2. **Raw 模式**：如果对象是 `TarsDict`、 `dict`、`list` 或基本类型，
       将进行原始编码（无 Schema）。

    Args:
        obj: 要编码的对象。

    Returns:
        包含序列化数据的 bytes 对象。

    Raises:
        TypeError: 如果对象既不是有效的 Struct 也不是支持的 Raw 类型。
        ValueError: 如果数据校验失败。
    """
    # 优先处理显式的 Raw 容器和基本类型
    if isinstance(
        obj, (TarsDict, dict, list, tuple, set, int, float, str, bytes, bool)
    ):
        return _core_encode_raw(obj)

    # 尝试作为 Struct 处理 (Struct)
    # 优化：通过检查特征属性避免 try-except 开销
    if isinstance(obj, Struct):
        return _core_encode(obj)

    # 如果不是 Struct，最后尝试 Raw 兜底
    return _core_encode_raw(obj)


@overload
def decode(
    data: _BytesLike,
    cls: type[TarsDict],
) -> TarsDict: ...


@overload
def decode(
    data: _BytesLike,
    cls: type[_StructT],
) -> _StructT: ...


def decode(
    data: _BytesLike,
    cls: type = TarsDict,
) -> Any:
    """从 Tars 二进制数据反序列化.

    Args:
        cls: 目标类
        data: 二进制数据

    Returns:
        反序列化的类实例或 TarsDict。

    Raises:
        TypeError: 参数类型错误或目标类未注册 Schema。
        ValueError: 数据格式不正确。
    """
    if issubclass(cls, (Struct)):
        return _core_decode(cls, data)
    else:
        return _core_decode_raw(data)



"""CLI 命令行工具集成测试.

验证 tarsio CLI 的核心功能:
- hex 字符串解码
- 文件输入处理
- 输出格式化
- 错误处理
"""

import json
from pathlib import Path

import pytest
from click.testing import CliRunner
from tarsio.__main__ import _create_cli


@pytest.fixture
def cli_runner() -> CliRunner:
    """提供 Click CLI 测试 runner.

    Returns:
        CliRunner: Click 测试 runner 实例.
    """
    return CliRunner()


@pytest.fixture
def cli():
    """提供 CLI Click Command 对象.

    Returns:
        Click Command.
    """
    return _create_cli()


# ==========================================
# 基本解码功能
# ==========================================


def test_cli_decode_hex_with_spaces(cli_runner: CliRunner, cli) -> None:
    """CLI 正确解码带空格的 hex 字符串."""
    result = cli_runner.invoke(cli, ["00 64"])
    assert result.exit_code == 0
    assert "100" in result.output


def test_cli_decode_hex_without_spaces(cli_runner: CliRunner, cli) -> None:
    """CLI 正确解码无空格的 hex 字符串."""
    result = cli_runner.invoke(cli, ["0064"])
    assert result.exit_code == 0
    assert "100" in result.output


def test_cli_decode_hex_with_0x_prefix(cli_runner: CliRunner, cli) -> None:
    """CLI 正确解码带 0x 前缀的 hex 字符串."""
    result = cli_runner.invoke(cli, ["0x0064"])
    assert result.exit_code == 0
    assert "100" in result.output


# ==========================================
# 输出格式
# ==========================================


def test_cli_format_json_outputs_valid_json(cli_runner: CliRunner, cli) -> None:
    """CLI --format json 输出有效 JSON."""
    result = cli_runner.invoke(cli, ["00 64", "--format", "json"])
    assert result.exit_code == 0
    # 输出包含 JSON 内容
    assert "100" in result.output


def test_cli_format_tree_outputs_tree_structure(cli_runner: CliRunner, cli) -> None:
    """CLI --format tree 输出树状结构."""
    result = cli_runner.invoke(cli, ["00 64", "--format", "tree"])
    assert result.exit_code == 0
    assert "(ROOT)" in result.output


def test_cli_format_pretty_is_default(cli_runner: CliRunner, cli) -> None:
    """CLI 默认使用 pretty 格式输出."""
    result = cli_runner.invoke(cli, ["00 64"])
    assert result.exit_code == 0


# ==========================================
# Verbose 模式
# ==========================================


def test_cli_verbose_shows_input_size(cli_runner: CliRunner, cli) -> None:
    """CLI -v 显示输入字节大小."""
    result = cli_runner.invoke(cli, ["00 64", "-v"])
    assert result.exit_code == 0
    assert "bytes" in result.output


def test_cli_verbose_shows_hex_dump(cli_runner: CliRunner, cli) -> None:
    """CLI --verbose 显示 hex dump."""
    result = cli_runner.invoke(cli, ["00 64", "--verbose"])
    assert result.exit_code == 0
    assert "00 64" in result.output


# ==========================================
# 文件输入
# ==========================================


def test_cli_read_binary_file(cli_runner: CliRunner, cli, tmp_path: Path) -> None:
    """CLI -f 正确读取二进制文件."""
    test_file = tmp_path / "test.bin"
    test_file.write_bytes(bytes.fromhex("0064"))

    result = cli_runner.invoke(cli, ["-f", str(test_file)])
    assert result.exit_code == 0
    assert "100" in result.output


def test_cli_read_binary_file_with_tree_format(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI -f 读取二进制文件并在 tree 模式下成功解码."""
    test_file = tmp_path / "test_tree.bin"
    test_file.write_bytes(bytes.fromhex("0064"))

    result = cli_runner.invoke(
        cli,
        ["-f", str(test_file), "--file-format", "bin", "--format", "tree"],
    )
    assert result.exit_code == 0
    assert "(ROOT)" in result.output


def test_cli_read_hex_text_file(cli_runner: CliRunner, cli, tmp_path: Path) -> None:
    """CLI -f 正确读取 hex 文本文件."""
    test_file = tmp_path / "test.hex"
    test_file.write_text("00 64")

    result = cli_runner.invoke(cli, ["-f", str(test_file), "--file-format", "hex"])
    assert result.exit_code == 0
    assert "100" in result.output


def test_cli_read_hex_text_file_with_invalid_utf8_returns_error(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI 读取非 UTF-8 hex 文本文件时返回错误."""
    test_file = tmp_path / "invalid_utf8.hex"
    test_file.write_bytes(b"\xff\xfe00")

    result = cli_runner.invoke(cli, ["-f", str(test_file), "--file-format", "hex"])
    assert result.exit_code != 0
    assert "输入读取失败" in result.output


def test_cli_read_hex_text_file_with_invalid_char_returns_error(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI 读取包含非法字符的 hex 文本文件时返回错误."""
    test_file = tmp_path / "invalid_char.hex"
    test_file.write_text("00 zz")

    result = cli_runner.invoke(cli, ["-f", str(test_file), "--file-format", "hex"])
    assert result.exit_code != 0
    assert "非法 hex 字符" in result.output


def test_cli_read_hex_text_file_with_odd_length_returns_error(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI 读取奇数长度 hex 文本文件时返回错误."""
    test_file = tmp_path / "odd_length.hex"
    test_file.write_text("0")

    result = cli_runner.invoke(cli, ["-f", str(test_file), "--file-format", "hex"])
    assert result.exit_code != 0
    assert "长度必须为偶数" in result.output


def test_cli_binary_file_handle_released_after_decode(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI 读取二进制文件后应释放句柄."""
    test_file = tmp_path / "handle.bin"
    test_file.write_bytes(bytes.fromhex("0064"))

    result = cli_runner.invoke(cli, ["-f", str(test_file), "--file-format", "bin"])
    assert result.exit_code == 0

    test_file.unlink()
    assert not test_file.exists()


# ==========================================
# 输出到文件
# ==========================================


def test_cli_output_to_file_saves_json(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI -o 正确保存 JSON 输出到文件."""
    output_file = tmp_path / "out.json"

    result = cli_runner.invoke(
        cli, ["00 64", "--format", "json", "-o", str(output_file)]
    )
    assert result.exit_code == 0

    data = json.loads(output_file.read_text())
    assert data["0"] == 100


def test_cli_output_to_directory_returns_error(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI 输出路径为目录时返回错误."""
    output_dir = tmp_path / "out_dir"
    output_dir.mkdir()

    result = cli_runner.invoke(cli, ["00 64", "-o", str(output_dir)])
    assert result.exit_code != 0
    assert "输出失败" in result.output


# ==========================================
# Probe 策略
# ==========================================


def test_cli_probe_off_does_not_expand_nested_structure(
    cli_runner: CliRunner, cli
) -> None:
    """CLI probe=off 时不展开嵌套结构."""
    nested_simplelist_hex = "0800010c1d0000020001"
    result = cli_runner.invoke(
        cli,
        [nested_simplelist_hex, "--format", "tree", "--probe", "off"],
    )
    assert result.exit_code == 0
    assert ">>> Probed Structure >>>" not in result.output


def test_cli_probe_on_expands_nested_structure(cli_runner: CliRunner, cli) -> None:
    """CLI probe=on 时展开嵌套结构."""
    nested_simplelist_hex = "0800010c1d0000020001"
    result = cli_runner.invoke(
        cli,
        [nested_simplelist_hex, "--format", "tree", "--probe", "on"],
    )
    assert result.exit_code == 0
    assert ">>> Probed Structure >>>" in result.output


def test_cli_probe_auto_skips_when_bytes_exceed_threshold(
    cli_runner: CliRunner, cli
) -> None:
    """CLI probe=auto 在超过阈值时跳过探测."""
    nested_simplelist_hex = "0800010c1d0000020001"
    result = cli_runner.invoke(
        cli,
        [
            nested_simplelist_hex,
            "--format",
            "tree",
            "--probe",
            "auto",
            "--probe-max-bytes",
            "1",
        ],
    )
    assert result.exit_code == 0
    assert ">>> Probed Structure >>>" not in result.output


# ==========================================
# 错误处理
# ==========================================


def test_cli_no_input_returns_error(cli_runner: CliRunner, cli) -> None:
    """CLI 无输入时返回错误."""
    result = cli_runner.invoke(cli, [])
    assert result.exit_code != 0


def test_cli_invalid_hex_returns_error(cli_runner: CliRunner, cli) -> None:
    """CLI 无效 hex 输入时返回错误."""
    result = cli_runner.invoke(cli, ["not-hex"])
    assert result.exit_code != 0


def test_cli_tree_trace_depth_exceeded_returns_error(
    cli_runner: CliRunner, cli
) -> None:
    """CLI tree 模式在 trace 深度超限时返回错误."""
    nested_hex = "0A" * 101 + "0B" * 101
    result = cli_runner.invoke(cli, [nested_hex, "--format", "tree"])
    assert result.exit_code != 0
    assert "解码失败" in result.output
    assert "Trace recursion depth exceeded" in result.output


def test_cli_both_inputs_returns_error(
    cli_runner: CliRunner, cli, tmp_path: Path
) -> None:
    """CLI 同时提供参数和文件时返回错误."""
    test_file = tmp_path / "test.bin"
    test_file.write_bytes(b"\x00\x64")

    result = cli_runner.invoke(cli, ["00 64", "-f", str(test_file)])
    assert result.exit_code != 0


# ==========================================
# 帮助信息
# ==========================================


def test_cli_help_shows_options(cli_runner: CliRunner, cli) -> None:
    """CLI --help 显示所有选项."""
    result = cli_runner.invoke(cli, ["--help"])
    assert result.exit_code == 0
    assert "--file" in result.output
    assert "--file-format" in result.output
    assert "--probe" in result.output
    assert "--probe-max-bytes" in result.output
    assert "--probe-max-depth" in result.output
    assert "--probe-max-nodes" in result.output
    assert "--format" in result.output
    assert "--verbose" in result.output



"""Special types benchmarks."""

import pytest
from tarsio._core import decode, encode

from .models import SpecialTypes


@pytest.mark.benchmark(group="special")
def test_bench_encode_special(benchmark, special_obj):
    """测试特殊类型(Union/Optional/Enum)编码性能."""
    benchmark(encode, special_obj)


@pytest.mark.benchmark(group="special")
def test_bench_decode_special(benchmark, special_bytes):
    """测试特殊类型(Union/Optional/Enum)解码性能."""
    benchmark(decode, SpecialTypes, special_bytes)



"""Tarsio 类型内省.

主要用途：

- 在开发阶段对 `typing.Annotated` 字段标注进行静态建模
- 提供 `type_info()` / `struct_info()` 的返回对象结构（`kind` 分支 + 关联字段）
"""

from typing import Any, TypeAlias, TypeVar

T = TypeVar("T")

class Type:
    """类型内省基类.

    Attributes:
        kind: 类型分支标识。
    """

    kind: str

class BasicType(Type):
    """基础标量类型基类."""

class CompoundType(Type):
    """复合容器类型基类."""

class IntType(BasicType):
    """整数类型（JCE int 家族的抽象视图）.

    编码：`ZeroTag` 或 `Int1/Int2/Int4/Int8`。
    """

    gt: float | None
    lt: float | None
    ge: float | None
    le: float | None

class StrType(BasicType):
    """字符串类型.

    编码：`String1` 或 `String4`。
    """

    min_length: int | None
    max_length: int | None
    pattern: str | None

class FloatType(BasicType):
    """浮点类型（运行时对应 double 语义）.

    编码：`ZeroTag` 或 `Double`。
    """

    gt: float | None
    lt: float | None
    ge: float | None
    le: float | None

class BoolType(BasicType):
    """布尔类型（在 JCE 编码层面通常以 int 表达）.

    编码：`ZeroTag` 或 `Int1/Int2/Int4/Int8`。
    """

class BytesType(BasicType):
    """二进制类型（运行时会被视为 byte-list 的特殊形式）.

    编码：`SimpleList`。
    """

    min_length: int | None
    max_length: int | None

class AnyType(BasicType):
    """动态类型（运行时根据值推断编码）.

    编码：运行时按值类型选择具体 TarsType。
    """

class NoneType(BasicType):
    """None 类型（通常仅出现在 Union/Optional 中）.

    编码：不能直接编码，仅用于 Optional/Union 的语义分支。
    """

class EnumType(CompoundType):
    """Enum 类型.

    编码：取 `value` 的内层类型映射。

    Attributes:
        cls: 枚举类型。
        value_type: 枚举值的类型内省结果。
    """

    cls: type
    value_type: TypeInfo

class UnionType(CompoundType):
    """Union 类型（非 Optional 形式）.

    编码：按变体顺序匹配实际值，直接按匹配类型编码。

    Attributes:
        variants: 变体类型列表。
    """

    variants: tuple[TypeInfo, ...]

class ListType(CompoundType):
    """列表类型：`list[T]`.

    编码：`List`（若元素类型为 int 且值为 bytes，则使用 `SimpleList`）。

    Attributes:
        item_type: 元素类型。
    """

    item_type: TypeInfo
    min_length: int | None
    max_length: int | None

class TupleType(CompoundType):
    """元组类型：固定长度、固定类型 `tuple[T1, T2, ...]`.

    编码：`List`。

    Attributes:
        items: 元素类型列表。
    """

    items: tuple[TypeInfo, ...]
    min_length: int | None
    max_length: int | None

class VarTupleType(CompoundType):
    """元组类型：可变长度、元素类型相同 `tuple[T, ...]`.

    编码：`List`（若元素类型为 int 且值为 bytes，则使用 `SimpleList`）。

    Attributes:
        item_type: 元素类型。
    """

    item_type: TypeInfo
    min_length: int | None
    max_length: int | None

class MapType(CompoundType):
    """映射类型：`dict[K, V]`.

    编码：`Map`。

    Attributes:
        key_type: 键类型。
        value_type: 值类型。
    """

    key_type: TypeInfo
    value_type: TypeInfo
    min_length: int | None
    max_length: int | None

class SetType(CompoundType):
    """集合类型：`set[T]` / `frozenset[T]`.

    编码：`List`，解码为 set。

    Attributes:
        item_type: 元素类型。
    """

    item_type: TypeInfo
    min_length: int | None
    max_length: int | None

class OptionalType(CompoundType):
    """可选类型：`T | None` 或 `typing.Optional[T]`.

    编码：None 时不写 tag，有值时按内层类型映射。

    Attributes:
        inner_type: 内层类型。
    """

    inner_type: TypeInfo

class StructType(CompoundType):
    """Struct 类型：字段类型为另一个 `tarsio.Struct` 子类.

    编码：`StructBegin` ... `StructEnd`。

    Attributes:
        cls: Struct 类型。
        fields: 字段列表，按 tag 升序。
    """

    cls: type
    fields: tuple[Field, ...]

class RefType(CompoundType):
    """引用类型：用于递归结构中的循环引用节点.

    Attributes:
        cls: 被引用的 Struct 类型。
    """

    cls: type

class TypedDictType(CompoundType):
    """TypedDict 类型.

    编码：`Map`。
    """

class NamedTupleType(CompoundType):
    """NamedTuple 类型.

    编码：`List`。

    Attributes:
        cls: NamedTuple 类型。
        items: 元素类型列表。
    """

    cls: type
    items: tuple[TypeInfo, ...]

class DataclassType(CompoundType):
    """Dataclass 类型.

    编码：`Map`。

    Attributes:
        cls: Dataclass 类型。
    """

    cls: type

class TarsDictType(CompoundType):
    """TarsDict 类型（动态 struct 字段映射）.

    编码：`StructBegin` ... `StructEnd`。
    """

TypeInfo: TypeAlias = (
    IntType
    | StrType
    | FloatType
    | BoolType
    | BytesType
    | AnyType
    | NoneType
    | TypedDictType
    | NamedTupleType
    | DataclassType
    | EnumType
    | UnionType
    | ListType
    | TupleType
    | VarTupleType
    | MapType
    | SetType
    | OptionalType
    | StructType
    | RefType
    | TarsDictType
)

class Field:
    """结构体字段信息.

    Attributes:
        name: 字段名。
        tag: 字段 tag。
        type: 字段类型内省结果。
        default: 字段默认值。
        default_factory: 字段默认值工厂；无工厂时为 `tarsio.NODEFAULT`。
        has_default: 是否显式有默认值。
        optional: 是否可选。
        required: 是否必填。
    """

    name: str
    tag: int
    type: TypeInfo
    default: Any
    default_factory: Any
    has_default: bool
    optional: bool
    required: bool

FieldInfo: TypeAlias = Field

class StructInfo:
    """结构体信息（类级 Schema 视图）.

    Attributes:
        cls: 结构体类型。
        fields: 字段列表，按 tag 升序。
    """

    cls: type
    fields: tuple[Field, ...]

def type_info(tp: Any) -> TypeInfo:
    """将类型标注解析为 Tarsio 的类型内省结果.

    Args:
        tp: 需要解析的类型标注，支持内置类型（如 `int/str/bytes`）、容器类型
            （如 `list[T]`、`tuple[T]`、`dict[K, V]`）、Optional/Union 形式，
            以及 `typing.Annotated[T, ...]`（会解析其中的 `Meta` 约束信息）。

    Returns:
        解析后的 `TypeInfo` 实例，可通过其 `kind` 字段区分具体分支，并读取对应属性。

    Raises:
        TypeError: 当类型标注不受支持或包含未支持的前向引用时抛出。
    """

def struct_info(cls: type) -> StructInfo | None:
    """解析 Struct 类并返回字段定义信息.

    Args:
        cls: 需要解析的 `tarsio.Struct` 子类。

    Returns:
        `StructInfo` 对象，包含字段列表（按 tag 升序）；如果该类没有可用字段，
        或该类是未具体化的泛型模板，则返回 `None`。

    Raises:
        TypeError: 当字段缺少 tag、tag 重复、混用整数 tag 与 `Meta`，或字段类型不受支持时抛出。
    """



"""Tarsio 公共 Python API.

本模块导出 Tarsio 的核心类型与编解码接口，包含基类 `Struct`、元类
`StructMeta`、配置对象 `StructConfig` 以及编码/解码函数。
"""

from collections.abc import Callable
from inspect import Signature
from typing import Any, ClassVar, Final, TypeVar, overload

from typing_extensions import dataclass_transform

from . import inspect

_StructT = TypeVar("_StructT")
_SM = TypeVar("_SM", bound="StructMeta")
_FieldDefaultT = TypeVar("_FieldDefaultT")
_BytesLike = bytes | bytearray | memoryview

__all__ = [
    "NODEFAULT",
    "Meta",
    "Struct",
    "StructConfig",
    "StructMeta",
    "TarsDict",
    "TraceNode",
    "ValidationError",
    "decode",
    "decode_raw",
    "decode_trace",
    "encode",
    "encode_raw",
    "field",
    "inspect",
    "probe_struct",
]

NODEFAULT: Final[object]

@overload
def field(
    *,
    tag: int | None = None,
    wrap_simplelist: bool = ...,
) -> Any: ...
@overload
def field(
    *,
    tag: int | None = None,
    default: Any,
    wrap_simplelist: bool = ...,
) -> Any: ...
@overload
def field(
    *,
    tag: int | None = None,
    wrap_simplelist: bool = ...,
    default_factory: Callable[[], _FieldDefaultT],
) -> _FieldDefaultT: ...
def field(
    *,
    tag: int | None = None,
    default: Any = NODEFAULT,
    wrap_simplelist: bool = False,
    default_factory: Any = NODEFAULT,
) -> Any:
    """声明字段默认值或默认值工厂.

    Args:
        tag: 字段 Tag, 范围 0-255。省略时自动分配。
        default: 字段默认值。
        wrap_simplelist: 是否将 Struct/TarsDict 字段包装为 SimpleList(bytes)。
            仅在字段注解为 Struct 或 TarsDict 时有效。
        default_factory: 字段默认值工厂（可调用对象）。

    Returns:
        内部字段规格对象。

    Raises:
        TypeError: 同时提供 default 与 default_factory，default_factory 不可调用，
            或 wrap_simplelist 非 bool 时抛出。
    """
    ...

class TarsDict(dict[int, Any]):
    """TarsDict 是一个特殊的字典类型，用于表示 Tars 结构中的 Tag-Value 映射.

    在 Tars 编解码过程中，TarsDict 用于存储和传递原始的 Tag-Value 数据，特别是在
    `encode_raw` 和 `decode_raw` 函数中。它的键是整数 Tag，值可以是任意类型（通常是
    基础类型、嵌套的 TarsDict 或列表）。

    Examples:
        ```python
        from tarsio import TarsDict, encode

        # 构造一个 TarsDict
        data = TarsDict(
            {
                0: 123,  # Tag 0: int
                1: "hello",  # Tag 1: str
                2: [1, 2, 3],  # Tag 2: list
            }
        )

        # 编码为 bytes
        encoded = encode(data)
        ```
    """

    ...

class ValidationError(ValueError):
    """解码阶段的校验错误.

    由 `Meta` 约束或 Schema 校验失败触发。

    错误消息包含路径与原因，典型格式：
    `Error at <root>.<field>.<tag:N>: <reason>`。

    Notes:
        当前版本不提供结构化字段访问器（如 `field_name`、`tag` 属性）。
        建议在业务侧解析异常消息中的路径片段。
    """

    ...

class Meta:
    """字段元数据与约束定义.

    用于在 `Annotated` 中提供运行时校验约束。

    Examples:
        ```python
        from typing import Annotated
        from tarsio import Struct, Meta, field

        class Product(Struct):
            # 价格必须 > 0
            price: Annotated[int, Meta(gt=0)] = field(tag=0)
            # 代码必须是 1-10 位大写字母
            code: Annotated[str, Meta(min_len=1, max_len=10, pattern=r"^[A-Z]+$")] = (
                field(tag=1)
            )
        ```
    """
    def __init__(
        self,
        gt: float | None = ...,
        lt: float | None = ...,
        ge: float | None = ...,
        le: float | None = ...,
        min_len: int | None = ...,
        max_len: int | None = ...,
        pattern: str | None = ...,
    ) -> None:
        """初始化字段元数据.

        Args:
            gt: 数值必须大于该值。
            lt: 数值必须小于该值。
            ge: 数值必须大于或等于该值。
            le: 数值必须小于或等于该值。
            min_len: 长度下限。
            max_len: 长度上限。
            pattern: 正则表达式约束。
        """
        ...

    gt: float | None
    lt: float | None
    ge: float | None
    le: float | None
    min_len: int | None
    max_len: int | None
    pattern: str | None

@dataclass_transform(
    eq_default=True,
    order_default=False,
    kw_only_default=False,
    frozen_default=False,
    field_specifiers=(field,),
)
class StructMeta(type):
    """Struct 的元类.

    用于在定义 `Struct` 子类时，在类创建期编译并注册 Schema，并生成相应的构造行为与
    运行时元信息（如 `__struct_fields__`、`__struct_config__`、`__signature__`）。
    """

    __struct_fields__: ClassVar[tuple[str, ...]]
    @property
    def __signature__(self) -> Signature: ...
    @property
    def __struct_config__(self) -> StructConfig: ...
    def __new__(
        mcls: type[_SM],
        name: str,
        bases: tuple[type, ...],
        namespace: dict[str, Any],
        /,
        *,
        frozen: bool = ...,
        order: bool = ...,
        forbid_unknown_tags: bool = ...,
        eq: bool = ...,
        omit_defaults: bool = ...,
        repr_omit_defaults: bool = ...,
        kw_only: bool = ...,
        dict: bool = ...,
        weakref: bool = ...,
        **kwargs: Any,
    ) -> _SM:
        """创建 Struct 子类并编译 Schema.

        Args:
            mcls: 元类本身。
            name: 类名。
            bases: 基类。
            namespace: 类命名空间。
            frozen: 是否冻结实例。
            order: 是否生成排序比较方法。
            forbid_unknown_tags: 是否禁止未知 Tag。
            eq: 是否生成相等比较。
            omit_defaults: 编码时是否省略默认值字段。
            repr_omit_defaults: repr 是否省略默认值字段。
            kw_only: 是否只允许关键字参数构造。
            dict: 是否为实例保留 `__dict__`。
            weakref: 是否支持弱引用。
            **kwargs: 预留扩展配置。

        Returns:
            新创建的 Struct 子类。
        """
        ...

class StructConfig:
    """Struct 的配置对象.

    该对象反映 `Struct` 子类在定义时传入的配置选项，可通过
    `Struct.__struct_config__` 或实例的 `__struct_config__` 访问。

    Attributes:
        frozen: 是否冻结实例（不可变）。
        eq: 是否启用值相等比较。
        order: 是否启用排序比较。
        kw_only: 构造函数是否仅接受关键字参数。
        repr_omit_defaults: `repr` 是否省略默认值字段。
        omit_defaults: 编码时是否省略默认值字段。
        weakref: 是否支持弱引用。
        dict: 是否保留 `__dict__`（允许动态属性）。
        rename: 预留字段（当前默认未启用）。
    """

    frozen: bool
    eq: bool
    order: bool
    kw_only: bool
    repr_omit_defaults: bool
    omit_defaults: bool
    weakref: bool
    dict: bool
    rename: Any | None

class Struct(metaclass=StructMeta):
    """高性能可序列化结构体基类.

    `Struct` 用于定义可编码/解码的 Tars/JCE 数据结构。字段通过类型注解声明，
    tag 支持显式与隐式混合：

    - 显式 tag：通过 `field(tag=...)` 指定。
    - 隐式 tag：未显式指定时按字段定义顺序自动分配。
      当未显式指定的字段位于显式 tag 字段之后时，会从该显式 tag 继续递增分配。

    `Annotated` 仅用于附加 `Meta` 约束，不再负责声明 tag。

    字段可以提供默认值。带默认值的字段在构造函数中表现为可选参数；当字段是 Optional 且未
    显式提供默认值时，其默认值视为 `None`。

    Struct 会提供/生成以下能力：

    - `__init__`：支持按 Tag 顺序的 positional 参数，以及按字段名的 keyword 参数。
    - `__eq__`：当 `eq=True` 时生成相等比较。
    - `__repr__`：生成可读的 repr；当 `repr_omit_defaults=True` 时省略默认值字段。
    - `__copy__`：生成浅拷贝。
    - `__post_init__`：若定义则在实例初始化完成后调用（包括解码路径）。
    - `__replace__`：返回替换指定字段后的新实例。
    - `__match_args__`：用于模式匹配的位置参数顺序。
    - `__rich_repr__`：为 rich pretty-print 提供字段迭代项。
    - 排序比较：当 `order=True` 时生成 `__lt__/__le__/__gt__/__ge__`。
    - Hash: 当 `frozen=True` 时提供 `__hash__`（使实例可哈希）。

    运行时元信息：

    - `__struct_fields__`：字段名元组，按 Tag 升序排列。
    - `__struct_config__`：配置对象（见 `StructConfig`）。

    Configuration:
        可在定义 `Struct` 子类时传入关键字参数控制行为：

        - frozen (bool, default False): 是否冻结实例。冻结后禁止属性赋值，并提供 `__hash__`。
        - order (bool, default False): 是否生成排序比较方法。
        - eq (bool, default True): 是否生成 `__eq__`。
        - kw_only (bool, default False): 是否将所有字段设为仅关键字参数。
        - omit_defaults (bool, default False): 编码时是否省略值等于默认值的字段。
        - repr_omit_defaults (bool, default False): repr 是否省略值等于默认值的字段。
        - forbid_unknown_tags (bool, default False): 解码时是否禁止出现未知 Tag.
        - dict (bool, default False): 是否为实例保留 `__dict__`（允许附加额外属性）。
        - weakref (bool, default False): 是否支持弱引用。

    Examples:
        基本用法：

        ```python
        from typing import Annotated
        from tarsio import Meta, Struct, field

        class User(Struct):
            uid: int = field(tag=0)
            name: str  # 自动分配 tag=1
            score: Annotated[int, Meta(ge=0)] = field(tag=2, default=0)

        user = User(uid=1, name="Ada")
        data = user.encode()
        restored = User.decode(data)
        assert restored == user
        ```

        启用配置项：

        ```python
        from tarsio import Struct, field

        class Point(Struct, frozen=True, order=True):
            x: int = field(tag=0)
            y: int = field(tag=1)
        ```
    """

    __struct_fields__: ClassVar[tuple[str, ...]]
    __struct_config__: ClassVar[StructConfig]
    __match_args__: ClassVar[tuple[str, ...]]

    def __init_subclass__(
        cls,
        *,
        frozen: bool = False,
        order: bool = False,
        forbid_unknown_tags: bool = False,
        eq: bool = True,
        omit_defaults: bool = False,
        repr_omit_defaults: bool = False,
        kw_only: bool = False,
        dict: bool = False,
        weakref: bool = False,
        **kwargs: Any,
    ) -> None:
        """配置 Struct 子类行为."""
        ...
    @classmethod
    def __class_getitem__(cls, params: Any) -> Any:
        """返回参数化后的 Struct 类型.

        - 未具体化模板允许实例化，TypeVar 会按 bound/constraints/Any 进行运行时解释。
        - 具体化参数（如 `Box[int]`）会生成可缓存复用的具体类型，并参与严格校验。
        - 若参数中仍包含未解析 TypeVar，则返回通用 GenericAlias 以支持继续组合泛型。
        """
        ...
    def encode(self) -> bytes:
        """将当前实例编码为 Tars 二进制数据.

        Returns:
            编码后的 bytes。

        Raises:
            ValueError: 缺少必填字段或类型不匹配。
        """
        ...
    @classmethod
    def decode(cls: type[_StructT], data: _BytesLike) -> _StructT:
        """将 Tars 二进制数据解码为当前类实例.

        Args:
            data: 待解码的 bytes。

        Returns:
            解码得到的实例。

        Raises:
            TypeError: 目标类未注册 Schema。
            ValueError: 数据格式不正确或缺少必填字段。
            ValidationError: 解码后 `__post_init__` 抛出 TypeError/ValueError。
        """
        ...
    def __replace__(self: _StructT, **changes: Any) -> _StructT:
        """返回替换部分字段后的新实例.

        Args:
            **changes: 需要替换的字段名和值。

        Returns:
            新实例，未替换字段沿用原实例值。

        Raises:
            TypeError: 包含未知字段名时抛出。
            ValidationError: 替换值不满足类型或约束时抛出。
        """
        ...
    def __rich_repr__(self) -> list[tuple[str, Any]]:
        """返回 rich pretty-print 使用的字段序列.

        Returns:
            形如 ``[(field_name, value), ...]`` 的字段序列，顺序按 tag。
        """
        ...

def encode(obj: Any) -> bytes:
    """将 Tars Struct 对象序列化为 Tars 二进制格式.

    Args:
        obj: `Struct`、dataclass、NamedTuple、TypedDict 的实例。

    Returns:
        包含序列化数据的 bytes 对象。

    Raises:
        TypeError: 如果对象不是有效的 Tars Struct。
    """
    ...

def decode(cls: type[_StructT], data: _BytesLike) -> _StructT:
    """从 Tars 二进制数据反序列化为类实例.

    Args:
        cls: 目标类（`Struct`、dataclass、NamedTuple、TypedDict）。
        data: 包含 Tars 编码数据的 bytes 对象。

    Returns:
        反序列化的类实例。

    Raises:
        TypeError: 如果类未注册 Schema。
        ValueError: 如果数据格式不正确。
    """
    ...

def encode_raw(obj: Any) -> bytes:
    """将对象编码为 Tars 二进制格式 (原始模式).

    如果输入是 `TarsDict`，则按结构体编码；否则按其自然类型（Map, List, Int 等）编码。
    Raw 模式下，`Struct` 实例在任意嵌套位置都允许编码为 Struct。

    Args:
        obj: 要编码的对象。

    Returns:
        编码后的字节对象。
    """
    ...

def decode_raw(data: _BytesLike) -> TarsDict:
    """将字节解码为 TarsDict.

    Args:
        data: 包含 Tars 编码数据的 bytes 对象。

    Returns:
        解码后的 TarsDict。
        Raw 模式下，任意嵌套层级的 StructBegin 都会还原为 TarsDict。
    """
    ...

def probe_struct(data: bytes) -> TarsDict | None:
    """尝试将字节数据递归解析为 Tars 结构.

    这是一个启发式工具，用于探测一段二进制数据是否恰好是有效的 Tars 序列化结构。
    它不仅检查格式，还会验证是否完全消费了数据。

    Args:
        data: 可能包含 Tars 结构的二进制数据。

    Returns:
        如果解析成功且数据完整，返回 TarsDict；否则返回 None。
    """
    ...

class TraceNode:
    """`decode_trace` 返回的调试树节点.

    Attributes:
        tag: 当前节点对应的 Tag。
        jce_type: JCE 类型名。
        value: 当前节点值（容器节点通常为 None）。
        children: 子节点列表。
        name: 字段名（有 Schema 时可用）。
        type_name: 类型名（有 Schema 时可用）。
        path: 从根开始的可读路径。
    """

    tag: int
    jce_type: str
    value: Any | None
    children: list[TraceNode]
    name: str | None
    type_name: str | None
    path: str

    def to_dict(self) -> dict[str, Any]: ...

def decode_trace(data: bytes, cls: type[Any] | None = None) -> TraceNode:
    """解析二进制数据并生成追踪树.

    Args:
        data: Tars 二进制数据.
        cls: 可选的 Struct 类型，用于提供 Schema 信息.

    Returns:
        根 TraceNode 对象.
    """
    ...



"""Raw direct encoding benchmarks."""

import pytest
from tarsio._core import encode_raw


@pytest.mark.benchmark(group="raw_direct")
def test_bench_raw_encode_direct_list(benchmark, raw_direct_list):
    """测试 Raw 模式直接编码 List (非 Struct)."""
    benchmark(encode_raw, raw_direct_list)


@pytest.mark.benchmark(group="raw_direct")
def test_bench_raw_encode_map_str_key(benchmark, raw_map_str_key):
    """测试 Raw 模式直接编码 Map (Str Key)."""
    benchmark(encode_raw, raw_map_str_key)



"""Benchmark for large struct decoding performance."""

from typing import Annotated

import pytest
from tarsio._core import Struct, decode, encode


# Define a struct with 20 fields of mixed types
class LargeStruct(Struct):
    """基准测试使用的大字段结构体."""

    f0: Annotated[int, 0]
    f1: Annotated[str, 1]
    f2: Annotated[bool, 2]
    f3: Annotated[int, 3]
    f4: Annotated[str, 4]
    f5: Annotated[bool, 5]
    f6: Annotated[int, 6]
    f7: Annotated[str, 7]
    f8: Annotated[bool, 8]
    f9: Annotated[int, 9]
    f10: Annotated[str, 10]
    f11: Annotated[bool, 11]
    f12: Annotated[int, 12]
    f13: Annotated[str, 13]
    f14: Annotated[bool, 14]
    f15: Annotated[int, 15]
    f16: Annotated[str, 16]
    f17: Annotated[bool, 17]
    f18: Annotated[int, 18]
    f19: Annotated[str, 19]


@pytest.fixture(scope="module")
def large_struct_data():
    """提供 LargeStruct 解码基准数据."""
    obj = LargeStruct(
        f0=100,
        f1="test_string_1",
        f2=True,
        f3=200,
        f4="test_string_2",
        f5=False,
        f6=300,
        f7="test_string_3",
        f8=True,
        f9=400,
        f10="test_string_4",
        f11=False,
        f12=500,
        f13="test_string_5",
        f14=True,
        f15=600,
        f16="test_string_6",
        f17=False,
        f18=700,
        f19="test_string_7",
    )
    return encode(obj)


@pytest.mark.benchmark(group="struct_decode")
def test_decode_large_struct(benchmark, large_struct_data):
    """基准测试：解码包含 20 个字段的结构体."""
    benchmark(decode, LargeStruct, large_struct_data)



"""Tarsio 基准测试共享 Fixtures."""

from typing import Any, cast

import pytest
from tarsio._core import TarsDict, encode

from tests.benchmarks.schema.models import (
    Containers,
    DeepNest,
    DerivedStruct,
    HighTagStruct,
    LargeData,
    MapEnumStruct,
    Medium,
    Mode,
    Primitives,
    SparseStruct,
    SpecialTypes,
    Status,
)


@pytest.fixture
def primitives_obj():
    """生成基础类型测试对象."""
    return Primitives(
        i8=10,
        i16=3000,
        i32=100000,
        i64=5000000000,
        f32=3.14,
        f64=2.718281828,
        s1="hello",
        s4="world" * 50,  # s4 > 255 length
        b=True,
    )


@pytest.fixture
def primitives_bytes(primitives_obj):
    """生成基础类型编码数据."""
    return encode(primitives_obj)


@pytest.fixture
def primitives_zeros():
    """全零值，测试 ZeroTag 压缩优化."""
    return Primitives(0, 0, 0, 0, 0.0, 0.0, "", "", False)


@pytest.fixture
def containers_obj(primitives_obj):
    """生成容器类型测试对象."""
    return Containers(
        lst_int=list(range(100)),
        lst_struct=[primitives_obj] * 10,
        mp_str_int={f"k{i}": i for i in range(50)},
        mp_int_str={i: f"v{i}" for i in range(50)},
        st_int=set(range(50)),
        tup=(1, "tuple", 2.0),
        vtup=(1, 2, 3, 4, 5),
    )


@pytest.fixture
def containers_bytes(containers_obj):
    """生成容器类型编码数据."""
    return encode(containers_obj)


@pytest.fixture
def medium_dict() -> TarsDict:
    """生成 Medium 对象的字典形式 (用于 Raw 对比)."""
    return TarsDict(
        {
            0: list(range(100)),
            1: [f"tag_{i}" for i in range(20)],
            2: {f"key_{i}": i for i in range(20)},
            3: None,
        }
    )


@pytest.fixture
def medium_struct_obj():
    """生成 Medium 结构体实例."""
    return Medium(
        ids=list(range(100)),
        tags=[f"tag_{i}" for i in range(20)],
        props={f"key_{i}": i for i in range(20)},
    )


@pytest.fixture
def medium_struct_bytes(medium_struct_obj):
    """生成 Medium 结构体编码数据."""
    return encode(medium_struct_obj)


@pytest.fixture
def special_obj():
    """生成特殊类型测试对象."""
    return SpecialTypes(
        opt_none=None,
        opt_val=123,
        u_int=456,
        u_str="union",
        e_int=Status.DONE,
        e_str=Mode.SAFE,
    )


@pytest.fixture
def special_bytes(special_obj):
    """生成特殊类型编码数据."""
    return encode(special_obj)


@pytest.fixture
def high_tag_obj():
    """生成高 Tag 测试对象."""
    return HighTagStruct(1, 2)


@pytest.fixture
def high_tag_bytes(high_tag_obj):
    """生成高 Tag 编码数据."""
    return encode(high_tag_obj)


@pytest.fixture
def large_data_obj():
    """生成大数据测试对象."""
    size = 10000
    return LargeData(blob=b"\x01" * size, ints=[1] * size)


@pytest.fixture
def large_data_bytes(large_data_obj):
    """生成大数据编码数据."""
    return encode(large_data_obj)


@pytest.fixture
def huge_blob_obj():
    """生成超大 Blob (10MB) 用于 Zero-copy 测试."""
    return LargeData(blob=b"\xff" * (10 * 1024 * 1024), ints=[])


@pytest.fixture
def huge_blob_bytes(huge_blob_obj):
    """生成超大 Blob 编码数据."""
    return encode(huge_blob_obj)


@pytest.fixture
def sparse_obj():
    """生成稀疏对象 (50个字段只填首尾)."""
    obj = cast(Any, SparseStruct())
    obj.f0 = 100
    obj.f49 = 200
    return obj


@pytest.fixture
def sparse_bytes(sparse_obj):
    """生成稀疏对象编码数据."""
    return encode(sparse_obj)


@pytest.fixture
def map_enum_obj():
    """生成 Map Enum Key 对象."""
    return MapEnumStruct(mapping={Status.INIT: 1, Status.DONE: 2, Status.ERROR: -1})


@pytest.fixture
def map_enum_bytes(map_enum_obj):
    """生成 Map Enum Key 编码数据."""
    return encode(map_enum_obj)


@pytest.fixture
def derived_obj():
    """生成继承结构对象."""
    return DerivedStruct(base_val=10, derived_val=20)


@pytest.fixture
def derived_bytes(derived_obj):
    """生成继承结构编码数据."""
    return encode(derived_obj)


@pytest.fixture
def malformed_bytes():
    """生成畸形数据 (截断)."""
    return b"\x00\x01\x02"


@pytest.fixture
def deep_nest_obj():
    """生成深度嵌套对象 (Depth 15)."""
    root = DeepNest(val=0)
    curr = root
    for i in range(1, 15):
        new_node = DeepNest(val=i)
        curr.next = new_node
        curr = new_node
    return root


@pytest.fixture
def deep_nest_bytes(deep_nest_obj):
    """生成深度嵌套编码数据."""
    return encode(deep_nest_obj)



"""Tarsio Raw benchmark payloads."""

import pytest
from tarsio._core import decode_raw, encode_raw


@pytest.mark.benchmark(group="raw_payloads")
def test_bench_raw_encode_huge_struct_blob(benchmark, raw_huge_blob_struct):
    """测试 Raw 模式编码 10MB Blob (Struct)."""
    benchmark(encode_raw, raw_huge_blob_struct)


@pytest.mark.benchmark(group="raw_payloads")
def test_bench_raw_decode_huge_struct_blob(benchmark, raw_huge_blob_bytes):
    """测试 Raw 模式解码 10MB Blob (Struct)."""
    benchmark(decode_raw, raw_huge_blob_bytes)



"""Benchmark for API dispatch overhead."""

from typing import Annotated

import pytest
import tarsio
import tarsio._core
from tarsio import Struct


class SimpleStruct(Struct):
    """基准测试使用的最小结构体."""

    id: Annotated[int, 0]


@pytest.fixture(scope="module")
def struct_obj():
    """提供 Struct 编码基准对象."""
    return SimpleStruct(id=123)


@pytest.fixture(scope="module")
def dict_obj():
    """提供 dict 编码基准对象."""
    return {1: 100, 2: "test"}


@pytest.fixture(scope="module")
def list_obj():
    """提供 list 编码基准对象."""
    return [1, 2, 3, 4, 5]


@pytest.fixture(scope="module")
def int_obj():
    """提供 int 编码基准对象."""
    return 12345


@pytest.mark.benchmark(group="dispatch_struct")
def test_encode_dispatch_struct(benchmark, struct_obj):
    """基准测试：tarsio.encode 对 Struct 的分发开销."""
    benchmark(tarsio.encode, struct_obj)


@pytest.mark.benchmark(group="dispatch_struct")
def test_encode_core_struct(benchmark, struct_obj):
    """基准测试：直接调用 tarsio._core.encode 处理 Struct."""
    benchmark(tarsio._core.encode, struct_obj)


@pytest.mark.benchmark(group="dispatch_dict")
def test_encode_dispatch_dict(benchmark, dict_obj):
    """基准测试：tarsio.encode 对 dict 的分发开销."""
    benchmark(tarsio.encode, dict_obj)


@pytest.mark.benchmark(group="dispatch_dict")
def test_encode_raw_dict(benchmark, dict_obj):
    """基准测试：直接调用 tarsio._core.encode_raw 处理 dict."""
    benchmark(tarsio._core.encode_raw, dict_obj)


@pytest.mark.benchmark(group="dispatch_list")
def test_encode_dispatch_list(benchmark, list_obj):
    """基准测试：tarsio.encode 对 list 的分发开销."""
    benchmark(tarsio.encode, list_obj)


@pytest.mark.benchmark(group="dispatch_list")
def test_encode_raw_list(benchmark, list_obj):
    """基准测试：直接调用 tarsio._core.encode_raw 处理 list."""
    benchmark(tarsio._core.encode_raw, list_obj)


@pytest.mark.benchmark(group="dispatch_int")
def test_encode_dispatch_int(benchmark, int_obj):
    """基准测试：tarsio.encode 对 int 的分发开销."""
    benchmark(tarsio.encode, int_obj)


@pytest.mark.benchmark(group="dispatch_int")
def test_encode_raw_int(benchmark, int_obj):
    """基准测试：直接调用 tarsio._core.encode_raw 处理 int."""
    benchmark(tarsio._core.encode_raw, int_obj)



"""Benchmark for raw map decoding performance."""

import pytest
from tarsio._core import decode_raw, encode_raw


@pytest.fixture(scope="module")
def map_int_keys_data():
    """提供 10k 整数键 map 的编码数据."""
    # 10,000 integer keys
    data = {i: i for i in range(10000)}
    return encode_raw(data)


@pytest.fixture(scope="module")
def map_str_keys_data():
    """提供 10k 字符串键 map 的编码数据."""
    # 10,000 string keys
    data = {str(i): i for i in range(10000)}
    return encode_raw(data)


@pytest.mark.benchmark(group="raw_map_decode")
def test_decode_map_int_keys_10k(benchmark, map_int_keys_data):
    """基准测试：解码 10k 整数键的原始 map."""
    benchmark(decode_raw, map_int_keys_data)


@pytest.mark.benchmark(group="raw_map_decode")
def test_decode_map_str_keys_10k(benchmark, map_str_keys_data):
    """基准测试：解码 10k 字符串键的原始 map."""
    benchmark(decode_raw, map_str_keys_data)



"""测试 typing/collections/generics 相关行为.

覆盖 Tarsio 支持的所有 Python 类型映射。
"""

import sys
from collections.abc import (
    Collection,
    Mapping,
    MutableMapping,
    MutableSequence,
    MutableSet,
    Sequence,
    Set,
)
from dataclasses import dataclass
from enum import Enum, IntEnum
from typing import (
    Annotated,
    Any,
    Final,
    Generic,
    Literal,
    NewType,
    Optional,
    TypeAlias,
    TypeVar,
    Union,
    cast,
)

import pytest
from tarsio._core import Meta, Struct, ValidationError, decode, encode, inspect
from typing_extensions import (
    NamedTuple,
    NotRequired,
    Required,
    TypeAliasType,
    TypedDict,
)

AliasInt: TypeAlias = int

# ==========================================
# 1. Primitives (基础类型)
# ==========================================


def test_primitives_roundtrip() -> None:
    """验证基础类型 (int, float, bool, str, bytes) 的编解码."""

    class PrimitiveStruct(Struct):
        i: Annotated[int, 0]
        f: Annotated[float, 1]
        b: Annotated[bool, 2]
        s: Annotated[str, 3]
        by: Annotated[bytes, 4]

    obj = PrimitiveStruct(123, 1.23, True, "hello", b"\x01\x02")
    encoded = encode(obj)
    decoded = decode(PrimitiveStruct, encoded)

    assert decoded.i == 123
    assert decoded.f == 1.23
    assert decoded.b is True
    assert decoded.s == "hello"
    assert decoded.by == b"\x01\x02"


# ==========================================
# 2. Containers (容器类型)
# ==========================================


def test_list_roundtrip() -> None:
    """验证 list[T] 的编解码."""

    class ListStruct(Struct):
        l1: Annotated[list[int], 0]
        l2: Annotated[list[str], 1]

    obj = ListStruct([1, 2], ["a", "b"])
    decoded = decode(ListStruct, encode(obj))
    assert decoded.l1 == [1, 2]
    assert decoded.l2 == ["a", "b"]


def test_tuple_fixed_roundtrip() -> None:
    """验证定长 tuple[T1, T2] 的编解码."""

    class TupleStruct(Struct):
        t: Annotated[tuple[int, str, float], 0]

    obj = TupleStruct((1, "a", 1.1))
    decoded = decode(TupleStruct, encode(obj))
    assert decoded.t == (1, "a", 1.1)


def test_tuple_variable_roundtrip() -> None:
    """验证变长 tuple[T, ...] 的编解码."""

    class VarTupleStruct(Struct):
        t: Annotated[tuple[int, ...], 0]

    obj = VarTupleStruct((1, 2, 3))
    decoded = decode(VarTupleStruct, encode(obj))
    assert decoded.t == (1, 2, 3)


def test_set_roundtrip() -> None:
    """验证 set[T] / frozenset[T] 的编解码."""

    class SetStruct(Struct):
        s: Annotated[set[int], 0]
        fs: Annotated[frozenset[int], 1]

    obj = SetStruct({1, 2}, frozenset({3, 4}))
    decoded = decode(SetStruct, encode(obj))
    assert decoded.s == {1, 2}
    assert decoded.fs == frozenset({3, 4})


def test_dict_roundtrip() -> None:
    """验证 dict[K, V] 的编解码."""

    class DictStruct(Struct):
        d1: Annotated[dict[str, int], 0]
        d2: Annotated[dict[int, str], 1]

    obj = DictStruct({"a": 1}, {2: "b"})
    decoded = decode(DictStruct, encode(obj))
    assert decoded.d1 == {"a": 1}
    assert decoded.d2 == {2: "b"}


# ==========================================
# 3. Abstract Base Classes (抽象基类)
# ==========================================


def test_abc_collection_list() -> None:
    """验证 Collection/Sequence 映射为 list."""

    class AbcListStruct(Struct):
        c: Annotated[Collection[int], 0]
        s: Annotated[Sequence[int], 1]
        ms: Annotated[MutableSequence[int], 2]
        ac: Annotated[Collection[int], 3]
        as_: Annotated[Sequence[int], 4]
        ams: Annotated[MutableSequence[int], 5]

    data = [1, 2]
    # cast necessary because Collection is abstract in typing but we pass list
    obj = AbcListStruct(data, data, data, data, data, data)  # type: ignore
    decoded = decode(AbcListStruct, encode(obj))

    for field in ["c", "s", "ms", "ac", "as_", "ams"]:
        assert getattr(decoded, field) == data
        assert isinstance(getattr(decoded, field), list)


def test_abc_collection_set() -> None:
    """验证 Set/MutableSet 映射为 set."""

    class AbcSetStruct(Struct):
        s: Annotated[Set[int], 0]
        ms: Annotated[MutableSet[int], 1]
        as_: Annotated[Set[int], 2]
        ams: Annotated[MutableSet[int], 3]

    data = {1, 2}
    obj = AbcSetStruct(data, data, data, data)  # type: ignore
    decoded = decode(AbcSetStruct, encode(obj))

    for field in ["s", "ms", "as_", "ams"]:
        assert getattr(decoded, field) == data
        assert isinstance(getattr(decoded, field), set)


def test_abc_collection_map() -> None:
    """验证 Mapping/MutableMapping 映射为 dict."""

    class AbcMapStruct(Struct):
        m: Annotated[Mapping[str, int], 0]
        mm: Annotated[MutableMapping[str, int], 1]
        am: Annotated[Mapping[str, int], 2]
        amm: Annotated[MutableMapping[str, int], 3]

    data = {"a": 1}
    obj = AbcMapStruct(data, data, data, data)  # type: ignore
    decoded = decode(AbcMapStruct, encode(obj))

    for field in ["m", "mm", "am", "amm"]:
        assert getattr(decoded, field) == data
        assert isinstance(getattr(decoded, field), dict)


# ==========================================
# 4. Structural Types (结构化类型)
# ==========================================


def test_dataclass_support() -> None:
    """验证标准库 dataclass 支持."""

    @dataclass
    class User:
        id: int
        name: str

    class Wrap(Struct):
        u: Annotated[User, 0]

    obj = Wrap(User(1, "dc"))
    decoded = decode(Wrap, encode(obj))
    assert decoded.u == User(1, "dc")


def test_namedtuple_support() -> None:
    """验证 NamedTuple 支持."""

    class Point(NamedTuple):
        x: int
        y: int

    class Wrap(Struct):
        p: Annotated[Point, 0]

    obj = Wrap(Point(10, 20))
    decoded = decode(Wrap, encode(obj))
    assert decoded.p == Point(10, 20)


def test_typeddict_support() -> None:
    """验证 TypedDict 支持 (含 Required/NotRequired)."""

    class UserDict(TypedDict):
        id: int
        name: NotRequired[str]

    class StrictDict(TypedDict):
        id: Required[int]

    class Wrap(Struct):
        u: Annotated[UserDict, 0]
        s: Annotated[StrictDict, 1]

    obj = Wrap({"id": 1, "name": "td"}, {"id": 2})
    decoded = decode(Wrap, encode(obj))
    assert decoded.u == {"id": 1, "name": "td"}
    assert decoded.s == {"id": 2}

    # Test NotRequired missing
    obj2 = Wrap({"id": 1}, {"id": 2})  # name missing
    decoded2 = decode(Wrap, encode(obj2))
    assert decoded2.u.get("name") is None


# ==========================================
# 5. Logic & Markers (逻辑与标记)
# ==========================================


def test_optional_union() -> None:
    """验证 Optional 和 Union."""

    class Logic(Struct):
        uni: Annotated[int | str, 0]
        opt: Annotated[int | None, 1]
        uni_opt: Annotated[int | str | None, 2]

    # Case 1: Values present
    obj1 = Logic("s", 1, 2)
    dec1 = decode(Logic, encode(obj1))
    assert dec1.uni == "s"
    assert dec1.opt == 1
    assert dec1.uni_opt == 2

    # Case 2: None
    obj2 = Logic(10, None, None)
    dec2 = decode(Logic, encode(obj2))
    assert dec2.uni == 10
    assert dec2.opt is None
    assert dec2.uni_opt is None


def test_any_type() -> None:
    """验证 Any 类型 (动态推断)."""

    class AnyStruct(Struct):
        val: Annotated[Any, 0]

    # Note: b"\x00" decodes to b"\x00" because auto_simplelist is removed
    cases = [
        (1, 1),
        ("s", "s"),
        (b"\x00", b"\x00"),  # Expect bytes, not str
        ([1, 2], [1, 2]),
        ({"a": 1}, {"a": 1}),
    ]
    for inp, exp in cases:
        obj = AnyStruct(inp)
        dec = decode(AnyStruct, encode(obj))
        assert dec.val == exp


def test_literal_type() -> None:
    """验证 Literal 类型."""

    class Lit(Struct):
        status: Annotated[Literal["ok", "err"], 0]
        code: Annotated[Literal[1, 2], 1]

    obj = Lit("ok", 1)
    dec = decode(Lit, encode(obj))
    assert dec.status == "ok"
    assert dec.code == 1


def test_newtype_final_alias() -> None:
    """验证 NewType, Final, TypeAlias."""
    MyInt = NewType("MyInt", int)

    class Wrapper(Struct):
        n: Annotated[MyInt, 0]
        f: Annotated[Final[int], 1]
        a: Annotated[AliasInt, 2]

    obj = Wrapper(MyInt(1), 2, 3)
    dec = decode(Wrapper, encode(obj))
    assert dec.n == 1
    assert dec.f == 2
    assert dec.a == 3


def test_generic_typevar_bound_runtime_validation() -> None:
    """未具体化 TypeVar(bound=...) 应按 bound 做运行时校验."""
    t_bound = TypeVar("t_bound", bound=int)

    class Box(Struct, Generic[t_bound]):
        value: Annotated[t_bound, 0]

    ok = Box(value=1)
    assert ok.value == 1

    with pytest.raises(ValidationError, match="type mismatch"):
        Box(value=cast(Any, "x"))


def test_generic_typevar_constraints_runtime_validation() -> None:
    """未具体化 TypeVar(constraints) 应按约束联合类型校验."""
    t_limited = TypeVar("t_limited", int, str)

    class Box(Struct, Generic[t_limited]):
        value: Annotated[t_limited, 0]

    assert Box(value=1).value == 1
    assert Box(value="x").value == "x"

    with pytest.raises(ValidationError, match="type mismatch"):
        Box(value=cast(Any, 1.2))


def test_parametrized_generic_struct_is_strict_and_cached() -> None:
    """具体化泛型应严格校验且相同参数命中缓存."""
    t_any = TypeVar("t_any")

    class Box(Struct, Generic[t_any]):
        value: Annotated[t_any, 0]

    box_int = Box[int]
    box_int_2 = Box[int]
    box_str = Box[str]

    assert box_int is box_int_2
    assert box_int is not box_str

    assert box_int(value=3).value == 3
    with pytest.raises(ValidationError, match="type mismatch"):
        box_int(value=cast(Any, "3"))


# ==========================================
# 6. Enum Types (枚举)
# ==========================================


def test_enum_types() -> None:
    """验证 Enum, IntEnum."""

    class Color(Enum):
        RED = 1
        BLUE = "blue"

    class Level(IntEnum):
        LOW = 10
        HIGH = 20

    class EnumStruct(Struct):
        c: Annotated[Color, 0]
        lvl: Annotated[Level, 1]

    obj = EnumStruct(Color.RED, Level.HIGH)
    dec = decode(EnumStruct, encode(obj))
    assert dec.c == Color.RED
    assert dec.lvl == Level.HIGH

    obj2 = EnumStruct(Color.BLUE, Level.LOW)
    dec2 = decode(EnumStruct, encode(obj2))
    assert dec2.c == Color.BLUE
    assert dec2.lvl == Level.LOW


if sys.version_info >= (3, 11):
    from enum import StrEnum

    def test_strenum() -> None:
        """验证 StrEnum (Py3.11+)."""

        class State(StrEnum):
            ON = "on"
            OFF = "off"

        class S(Struct):
            s: Annotated[State, 0]

        obj = S(State.ON)
        dec = decode(S, encode(obj))
        assert dec.s == State.ON


# ==========================================
# 7. Inspect API (类型内省)
# ==========================================


def test_inspect_type_info_supported_kinds() -> None:
    """验证 type_info 覆盖所有支持类型."""

    class Inner(Struct):
        v: Annotated[int, 0]

    class Level(IntEnum):
        LOW = 1
        HIGH = 2

    cases = {
        "int": inspect.type_info(int),
        "str": inspect.type_info(str),
        "float": inspect.type_info(float),
        "bool": inspect.type_info(bool),
        "bytes": inspect.type_info(bytes),
        "any": inspect.type_info(Any),
        "none": inspect.type_info(type(None)),
        "enum": inspect.type_info(Level),
        "union": inspect.type_info(Union[int, str]),  # noqa: UP007
        "list": inspect.type_info(list[int]),
        "tuple": inspect.type_info(tuple[int, str]),
        "var_tuple": inspect.type_info(tuple[int, ...]),
        "map": inspect.type_info(dict[str, int]),
        "set": inspect.type_info(set[int]),
        "optional": inspect.type_info(Optional[int]),  # noqa: UP045
        "struct": inspect.type_info(Inner),
    }

    assert cases["int"].kind == "int"
    assert cases["str"].kind == "str"
    assert cases["float"].kind == "float"
    assert cases["bool"].kind == "bool"
    assert cases["bytes"].kind == "bytes"
    assert cases["any"].kind == "any"
    assert cases["none"].kind == "none"
    assert cases["enum"].kind == "enum"
    assert cases["union"].kind == "union"
    assert cases["list"].kind == "list"
    assert cases["tuple"].kind == "tuple"
    assert cases["var_tuple"].kind == "var_tuple"
    assert cases["map"].kind == "map"
    assert cases["set"].kind == "set"
    assert cases["optional"].kind == "optional"
    assert cases["struct"].kind == "struct"

    enum_info = cast(inspect.EnumType, cases["enum"])
    struct_info = cast(inspect.StructType, cases["struct"])
    assert enum_info.value_type.kind == "int"
    assert struct_info.cls is Inner
    assert isinstance(cases["int"], inspect.Type)
    assert isinstance(cases["int"], inspect.BasicType)
    assert isinstance(cases["list"], inspect.CompoundType)


def test_inspect_struct_type_contains_fields_tree() -> None:
    """验证 StructType 提供 fields 递归类型树."""

    class Inner(Struct):
        v: Annotated[int, 0]

    class Outer(Struct):
        item: Annotated[Inner, 0]

    info = cast(inspect.StructType, inspect.type_info(Outer))
    assert info.kind == "struct"
    assert [f.name for f in info.fields] == ["item"]
    assert info.fields[0].type.kind == "struct"
    assert cast(inspect.StructType, info.fields[0].type).cls is Inner


def test_inspect_struct_type_cycle_uses_ref_type() -> None:
    """验证递归结构在循环位置返回 RefType."""

    class Node(Struct):
        val: Annotated[int, 0]
        next: Annotated[Optional["Node"], 1] = None

    info = cast(inspect.StructType, inspect.type_info(Node))
    assert info.kind == "struct"
    next_type = cast(inspect.OptionalType, info.fields[1].type)
    assert next_type.kind == "optional"
    assert next_type.inner_type.kind == "ref"
    assert cast(inspect.RefType, next_type.inner_type).cls is Node


def test_inspect_struct_info_fields() -> None:
    """验证 struct_info 的字段结构与属性."""

    class Sample(Struct):
        a: Annotated[int, 0]
        b: Annotated[str, 1] = "x"
        c: Annotated[int | None, 2] = None

    info = inspect.struct_info(Sample)
    assert info is not None
    assert info.cls is Sample
    assert [f.name for f in info.fields] == ["a", "b", "c"]

    field_a, field_b, field_c = info.fields
    assert field_a.tag == 0
    assert field_a.type.kind == "int"
    assert field_a.required is True

    assert field_b.tag == 1
    assert field_b.type.kind == "str"
    assert field_b.has_default is True

    assert field_c.tag == 2
    assert field_c.type.kind == "optional"
    assert field_c.optional is True


def test_inspect_struct_info_plain_annotations_auto_tag() -> None:
    """验证全普通注解字段在 struct_info 中自动分配 tag."""

    class Sample(Struct):
        a: int
        b: str

    info = inspect.struct_info(Sample)
    assert info is not None
    assert [f.tag for f in info.fields] == [0, 1]
    assert [f.type.kind for f in info.fields] == ["int", "str"]


def test_inspect_constraints_from_meta() -> None:
    """验证 Meta 约束进入 constraints."""

    class Limited(Struct):
        v: Annotated[int, Meta(gt=1, le=10)]

    info = inspect.struct_info(Limited)
    assert info is not None
    field = info.fields[0]
    int_type = cast(inspect.IntType, field.type)
    assert int_type.gt == 1
    assert int_type.le == 10


def test_inspect_direct_constraints_for_str_type() -> None:
    """验证字符串类型可直接访问约束属性."""

    class Limited(Struct):
        name: Annotated[str, Meta(min_len=1, max_len=8, pattern=r"^[a-z]+$")]

    info = inspect.struct_info(Limited)
    assert info is not None
    str_type = cast(inspect.StrType, info.fields[0].type)
    assert str_type.min_length == 1
    assert str_type.max_length == 8
    assert str_type.pattern == r"^[a-z]+$"


AliasInt2 = TypeAliasType("AliasInt2", int)


def test_inspect_type_alias_type() -> None:
    """验证 TypeAliasType 能被解析."""
    info = inspect.type_info(AliasInt2)
    assert info.kind == "int"



from typing import Annotated, Any, Optional, cast

from tarsio._core import (
    NODEFAULT,
    Struct,
    TarsDict,
    decode,
    decode_raw,
    encode,
    encode_raw,
    field,
    inspect,
    probe_struct,
)
from typing_extensions import assert_type


class User(Struct):
    """用户结构体示例."""

    uid: Annotated[int, 0]
    name: Annotated[str, 1]


def test_type_decode() -> None:
    """验证 decode 与 Struct.decode 的返回类型推导."""
    # 使用有效的编码数据进行 User 结构体测试

    user_obj = User(uid=123, name="test_user")

    data = encode(user_obj)

    # 测试顶层 decode 函数

    user = decode(User, data)

    assert_type(user, User)

    # 测试 Struct.decode 类方法

    user2 = User.decode(data)

    assert_type(user2, User)


def test_type_encode() -> None:
    """验证 encode 与 Struct.encode 的返回类型推导."""
    user = User(uid=1, name="test")

    # 测试顶层 encode 函数

    data = encode(user)

    assert_type(data, bytes)

    # 测试实例的 encode 方法

    data2 = user.encode()

    assert_type(data2, bytes)


def test_type_raw() -> None:
    """验证原始编解码接口的 TarsDict 类型一致性."""
    raw_data: TarsDict = TarsDict({0: 123, 1: "hello"})

    # 编码为字节

    encoded = encode_raw(raw_data)

    assert_type(encoded, bytes)

    # 解码回 TarsDict

    decoded = decode_raw(encoded)

    assert_type(decoded, TarsDict)


def test_type_tars_dict_usage() -> None:
    """验证 TarsDict 的基本使用类型."""
    d = TarsDict({0: 1, 1: "s"})

    assert_type(d, TarsDict)

    assert_type(d[0], Any)


def test_type_containers() -> None:
    """验证容器类型的编解码类型推导."""
    # 列表测试 (Raw 模式)

    lst = [1, 2, 3]

    enc_lst = encode_raw(lst)

    assert_type(enc_lst, bytes)

    # 字典 (Map) 测试 (Raw 模式)

    mp = {"a": 1}

    enc_mp = encode_raw(mp)

    assert_type(enc_mp, bytes)


def test_type_probe_struct() -> None:
    """验证 probe_struct 返回类型."""
    data = b"..."

    res = probe_struct(data)

    # probe_struct 返回 TarsDict | None

    assert_type(res, TarsDict | None)


def test_type_inspect_hierarchy() -> None:
    """验证 inspect 类型树与别名的静态类型."""

    class Node(Struct):
        val: Annotated[int, 0]
        next: Annotated[Optional["Node"], 1] = None

    int_info = cast(inspect.IntType, inspect.type_info(int))
    node_info = cast(inspect.StructType, inspect.type_info(Node))

    assert_type(int_info, inspect.IntType)
    assert_type(node_info, inspect.StructType)
    assert_type(node_info.fields[0], inspect.Field)
    assert_type(node_info.fields[0], inspect.FieldInfo)
    assert_type(int_info.ge, float | None)


def test_type_field_and_nodefault() -> None:
    """验证 field 与 NODEFAULT 的静态类型可用性."""

    class Box(Struct):
        val: Annotated[list[int], 0] = field(default_factory=list)

    b = Box()
    assert_type(b.val, list[int])
    assert_type(NODEFAULT, object)



"""Tarsio 基准测试共享模型定义."""

from enum import Enum, IntEnum
from typing import Annotated, Optional

from tarsio import Struct


class Status(IntEnum):
    """整数枚举."""

    INIT = 0
    RUNNING = 1
    DONE = 2
    ERROR = 3


class Mode(str, Enum):
    """字符串枚举."""

    FAST = "fast"
    SAFE = "safe"


class Primitives(Struct):
    """覆盖所有基础类型 (ZeroTag 优化路径 + 正常值)."""

    i8: Annotated[int, 0]
    i16: Annotated[int, 1]
    i32: Annotated[int, 2]
    i64: Annotated[int, 3]
    f32: Annotated[float, 4]
    f64: Annotated[float, 5]
    s1: Annotated[str, 6]
    s4: Annotated[str, 7]
    b: Annotated[bool, 8]


class Containers(Struct):
    """覆盖所有容器类型."""

    lst_int: Annotated[list[int], 0]
    lst_struct: Annotated[list[Primitives], 1]
    mp_str_int: Annotated[dict[str, int], 2]
    mp_int_str: Annotated[dict[int, str], 3]
    st_int: Annotated[set[int], 4]
    tup: Annotated[tuple[int, str, float], 5]  # Fixed tuple
    vtup: Annotated[tuple[int, ...], 6]  # Var tuple


class SpecialTypes(Struct):
    """覆盖特殊类型 (Union, Optional, Enum)."""

    opt_none: Annotated[int | None, 0]
    opt_val: Annotated[int | None, 1]
    u_int: Annotated[int | str, 2]
    u_str: Annotated[int | str, 3]
    e_int: Annotated[Status, 4]
    e_str: Annotated[Mode, 5]


class HighTagStruct(Struct):
    """测试高 Tag (>15) 的头部编码."""

    val_low: Annotated[int, 0]
    val_high: Annotated[int, 200]


class Medium(Struct):
    """中等规模结构体 (容器)."""

    ids: Annotated[list[int], 0]
    tags: Annotated[list[str], 1]
    props: Annotated[dict[str, int], 2]
    note: Annotated[str | None, 3] = None


class DeepNest(Struct):
    """测试递归深度 (轻量级，只测开销)."""

    val: Annotated[int, 0]
    next: Annotated[Optional["DeepNest"], 1] = None


class LargeData(Struct):
    """大数据测试."""

    blob: Annotated[bytes, 0]  # SimpleList
    ints: Annotated[list[int], 1]  # Standard List


# 动态创建 SparseStruct 以包含 50 个可选字段
_sparse_annotations = {f"f{i}": Annotated[int | None, i] for i in range(50)}
_sparse_defaults = {f"f{i}": None for i in range(50)}
SparseStruct = type(
    "SparseStruct",
    (Struct,),
    {
        "__annotations__": _sparse_annotations,
        "__module__": __name__,
        **_sparse_defaults,
    },
)


class MapEnumStruct(Struct):
    """测试 Enum 作为 Map Key."""

    mapping: Annotated[dict[Status, int], 0]


class BaseStruct(Struct):
    """基类."""

    base_val: Annotated[int, 0]


class DerivedStruct(BaseStruct):
    """派生类."""

    derived_val: Annotated[int, 1]



from dataclasses import dataclass

from tarsio import encode


@dataclass
class U1:
    """测试用 Union 变体 1."""

    x: int


@dataclass
class U2:
    """测试用 Union 变体 2."""

    y: int


@dataclass
class U3:
    """测试用 Union 变体 3."""

    z: str


@dataclass
class U4:
    """测试用 Union 变体 4."""

    a: bool


@dataclass
class U5:
    """测试用 Union 变体 5."""

    b: float


@dataclass
class Wrapper:
    """含有 5 个变体的 Union 容器."""

    val: U1 | U2 | U3 | U4 | U5


def test_union_encoding_perf(benchmark):
    """验证 Union 类型在特定变体命中时的编码性能."""
    # 测试场景：编码最后一个变体，触发初始线性扫描但在缓存命中后降至 O(1).
    obj = Wrapper(val=U5(b=3.14))

    def run_encode():
        encode(obj)

    benchmark(run_encode)


def test_union_encoding_mixed_perf(benchmark):
    """验证 Union 类型在混合变体场景下的缓存压力与分发性能."""
    # 测试场景：混合多种变体以压测缓存的并发读取与分发稳定性.
    objs = [
        Wrapper(val=U1(x=1)),
        Wrapper(val=U2(y=2)),
        Wrapper(val=U3(z="hello")),
        Wrapper(val=U4(a=True)),
        Wrapper(val=U5(b=3.14)),
    ]

    def run_encode_mixed():
        for obj in objs:
            encode(obj)

    benchmark(run_encode_mixed)



"""Structural benchmarks (Recursion, Inheritance)."""

from typing import Annotated

import pytest
from tarsio._core import Struct, decode, encode

from .models import DeepNest, DerivedStruct


@pytest.mark.benchmark(group="recursion")
def test_bench_encode_deep_nest(benchmark, deep_nest_obj):
    """测试深度递归结构编码."""
    benchmark(encode, deep_nest_obj)


@pytest.mark.benchmark(group="recursion")
def test_bench_decode_deep_nest(benchmark, deep_nest_bytes):
    """测试深度递归结构解码."""
    benchmark(decode, DeepNest, deep_nest_bytes)


@pytest.mark.benchmark(group="inheritance")
def test_bench_encode_inheritance(benchmark, derived_obj):
    """测试继承结构体编码性能."""
    benchmark(encode, derived_obj)


@pytest.mark.benchmark(group="inheritance")
def test_bench_decode_inheritance(benchmark, derived_bytes):
    """测试继承结构体解码性能."""
    benchmark(decode, DerivedStruct, derived_bytes)


def _make_struct(size: int) -> tuple[type[Struct], dict[str, int]]:
    annotations = {f"f{i}": Annotated[int, i] for i in range(size)}
    payload = {f"f{i}": i for i in range(size)}

    cls = type(
        f"Init{size}",
        (Struct,),
        {"__annotations__": annotations, "__module__": __name__},
    )
    return cls, payload


INIT_8, PAYLOAD_8 = _make_struct(8)
INIT_24, PAYLOAD_24 = _make_struct(24)
INIT_64, PAYLOAD_64 = _make_struct(64)


@pytest.mark.benchmark(group="init_small")
def test_bench_init_small(benchmark):
    """测试小结构体 __init__ 性能."""
    benchmark(INIT_8, **PAYLOAD_8)


@pytest.mark.benchmark(group="init_medium")
def test_bench_init_medium(benchmark):
    """测试中结构体 __init__ 性能."""
    benchmark(INIT_24, **PAYLOAD_24)


@pytest.mark.benchmark(group="init_large")
def test_bench_init_large(benchmark):
    """测试大结构体 __init__ 性能."""
    benchmark(INIT_64, **PAYLOAD_64)



"""Tarsio Raw benchmark nested data."""

import pytest
from tarsio._core import decode_raw, encode_raw


@pytest.mark.benchmark(group="raw_nested")
def test_bench_raw_encode_nested(benchmark, raw_data_nested):
    """测试 Raw 模式编码嵌套字典."""
    benchmark(encode_raw, raw_data_nested)


@pytest.mark.benchmark(group="raw_nested")
def test_bench_raw_decode_nested(benchmark, raw_bytes_nested):
    """测试 Raw 模式解码嵌套字典."""
    benchmark(decode_raw, raw_bytes_nested)



"""Tarsio Raw benchmark mixed data."""

import pytest
from tarsio._core import decode_raw, encode_raw


@pytest.mark.benchmark(group="raw_mixed")
def test_bench_raw_encode_mixed(benchmark, raw_mixed_data):
    """测试 Raw 模式混合数据编码性能."""
    benchmark(encode_raw, raw_mixed_data)


@pytest.mark.benchmark(group="raw_mixed")
def test_bench_raw_decode_mixed(benchmark, raw_mixed_bytes):
    """测试 Raw 模式混合数据解码性能."""
    benchmark(decode_raw, raw_mixed_bytes)



"""Tarsio Raw benchmark sparse tag."""

import pytest
from tarsio._core import encode_raw


@pytest.mark.benchmark(group="raw_sparse")
def test_bench_raw_encode_sparse_tag(benchmark, raw_sparse_tag):
    """测试 Raw 模式编码稀疏 Tag (Tag 250)."""
    benchmark(encode_raw, raw_sparse_tag)



"""Containers benchmarks."""

import pytest
from tarsio._core import decode, encode

from .models import Containers, MapEnumStruct, SparseStruct


@pytest.mark.benchmark(group="containers")
def test_bench_encode_containers(benchmark, containers_obj):
    """测试容器类型编码性能."""
    benchmark(encode, containers_obj)


@pytest.mark.benchmark(group="containers")
def test_bench_decode_containers(benchmark, containers_bytes):
    """测试容器类型解码性能."""
    benchmark(decode, Containers, containers_bytes)


@pytest.mark.benchmark(group="sparse")
def test_bench_encode_sparse(benchmark, sparse_obj):
    """测试稀疏字段结构体编码."""
    benchmark(encode, sparse_obj)


@pytest.mark.benchmark(group="sparse")
def test_bench_decode_sparse(benchmark, sparse_bytes):
    """测试稀疏字段结构体解码."""
    benchmark(decode, SparseStruct, sparse_bytes)


@pytest.mark.benchmark(group="map_complex")
def test_bench_encode_map_enum(benchmark, map_enum_obj):
    """测试 Enum 作为 Map Key 的编码性能."""
    benchmark(encode, map_enum_obj)


@pytest.mark.benchmark(group="map_complex")
def test_bench_decode_map_enum(benchmark, map_enum_bytes):
    """测试 Enum 作为 Map Key 的解码性能."""
    benchmark(decode, MapEnumStruct, map_enum_bytes)



"""Tarsio Raw 基准测试共享 Fixtures."""

import pytest
from tarsio._core import TarsDict, encode_raw


@pytest.fixture
def raw_data_small() -> TarsDict:
    """生成 Small 对应的 raw dict."""
    return TarsDict({0: 123, 1: "benchmark", 2: True, 3: 3.14159})


@pytest.fixture
def raw_bytes_small(raw_data_small):
    """生成 raw dict 编码后的 bytes."""
    return encode_raw(raw_data_small)


@pytest.fixture
def raw_data_nested() -> TarsDict:
    """生成 Parent 对应的 raw dict (复杂嵌套)."""
    children = [TarsDict({0: i, 1: f"child_{i}"}) for i in range(50)]
    return TarsDict({0: children, 1: {"env": "production", "version": "1.0.0"}})


@pytest.fixture
def raw_bytes_nested(raw_data_nested):
    """生成 raw nested dict 编码后的 bytes."""
    return encode_raw(raw_data_nested)


@pytest.fixture
def raw_mixed_data() -> TarsDict:
    """生成混合类型字典."""
    return TarsDict(
        {0: 123, 1: "hello", 2: [1, 2, 3], 3: TarsDict({10: "nest"}), 200: 999}
    )


@pytest.fixture
def raw_mixed_bytes(raw_mixed_data):
    """生成混合类型编码数据."""
    return encode_raw(raw_mixed_data)


@pytest.fixture
def raw_huge_blob_struct() -> TarsDict:
    """生成包含 10MB Blob 的 Raw Struct (字典)."""
    return TarsDict({0: b"\x00" * (10 * 1024 * 1024)})


@pytest.fixture
def raw_huge_blob_bytes(raw_huge_blob_struct):
    """生成包含 10MB Blob 的 Raw Struct 编码数据."""
    return encode_raw(raw_huge_blob_struct)


@pytest.fixture
def raw_direct_list():
    """生成直接列表 (非 Struct)."""
    return [1, 2, 3, 4, 5]


@pytest.fixture
def raw_map_str_key():
    """生成字符串 Key 的字典 (Map, 非 Struct)."""
    return {"a": 1, "b": 2}


@pytest.fixture
def raw_sparse_tag() -> TarsDict:
    """生成稀疏 Tag 的 Struct."""
    return TarsDict({250: "sparse"})


@pytest.fixture
def raw_truncated_bytes():
    """生成截断的数据."""
    return b"\x02"  # Tag 0, Int4 (needs 4 bytes)... but EOF immediately



"""Comparison benchmarks."""

import pytest
from tarsio._core import decode, decode_raw, encode, encode_raw

from .models import Medium


@pytest.mark.benchmark(group="compare_encode")
def test_bench_compare_schema_encode(benchmark, medium_struct_obj):
    """[对比] Schema 模式编码 Medium 结构."""
    benchmark(encode, medium_struct_obj)


@pytest.mark.benchmark(group="compare_encode")
def test_bench_compare_raw_encode(benchmark, medium_dict):
    """[对比] Raw 模式编码 Medium 字典."""
    benchmark(encode_raw, medium_dict)


@pytest.mark.benchmark(group="compare_decode")
def test_bench_compare_schema_decode(benchmark, medium_struct_bytes):
    """[对比] Schema 模式解码 Medium 结构."""
    benchmark(decode, Medium, medium_struct_bytes)


@pytest.mark.benchmark(group="compare_decode")
def test_bench_compare_raw_decode(benchmark, medium_struct_bytes):
    """[对比] Raw 模式解码 Medium 数据."""
    benchmark(decode_raw, medium_struct_bytes)



"""Tarsio CLI - Tars 编解码命令行工具."""

from __future__ import annotations

import json
import mmap
import string
import sys
from dataclasses import dataclass, field
from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    import click as click_module
    from rich.console import Console as ConsoleType
    from rich.tree import Tree as TreeType
else:
    try:
        import click as click_module
        from rich.console import Console as ConsoleType
        from rich.tree import Tree as TreeType
    except ImportError:
        click_module = None
        ConsoleType = None
        TreeType = None

from tarsio._core import TraceNode, decode_raw, decode_trace, probe_struct

click = click_module


@dataclass(slots=True)
class ProbePolicy:
    """深度探测策略配置."""

    mode: str
    max_bytes: int
    max_depth: int
    max_nodes: int


@dataclass(slots=True)
class ProbeRuntime:
    """一次 CLI 执行的探测运行时状态."""

    probed_nodes: int = 0
    probe_cache: dict[bytes, Any | None] = field(default_factory=dict)
    trace_cache: dict[bytes, TraceNode] = field(default_factory=dict)


@dataclass(slots=True)
class InputBuffer:
    """输入缓冲区与其资源句柄."""

    data: bytes | memoryview
    mm: mmap.mmap | None = None
    view: memoryview | None = None

    def close(self) -> None:
        """释放输入资源."""
        if self.view is not None:
            self.view.release()
            self.view = None
        if self.mm is not None:
            self.mm.close()
            self.mm = None


def _check_cli_deps() -> None:
    """检查 CLI 依赖是否安装."""
    if not click:
        print(
            "错误: CLI 依赖未安装\n请运行: pip install tarsio[cli]",
            file=sys.stderr,
        )
        sys.exit(1)


def parse_hex_string(hex_str: str) -> bytes:
    """解析 hex 字符串为字节.

    Args:
        hex_str: hex 编码字符串.

    Returns:
        解析后的字节数据.
    """
    cleaned = hex_str.strip()
    if cleaned.lower().startswith("0x"):
        cleaned = cleaned[2:]
    return bytes.fromhex(cleaned)


def _parse_hex_stream(path: Path, chunk_size: int = 65536) -> bytes:
    """流式解析 hex 文本文件.

    Args:
        path: hex 文本文件路径.
        chunk_size: 单次读取字符数.

    Returns:
        解析后的字节数据.

    Raises:
        UnicodeDecodeError: 文件不是 UTF-8 文本.
        ValueError: 文件内容不是合法 hex.
        OSError: 文件读取失败.
    """
    hex_digits = set(string.hexdigits)
    output = bytearray()
    pending: str | None = None
    saw_digit = False
    pending_prefix_zero = False
    pending_prefix_pos = -1
    index = 0

    def push_hex(ch: str, pos: int) -> None:
        nonlocal pending, saw_digit
        if ch not in hex_digits:
            raise ValueError(f"第 {pos} 位包含非法 hex 字符: {ch!r}")
        saw_digit = True
        if pending is None:
            pending = ch
        else:
            output.append(int(f"{pending}{ch}", 16))
            pending = None

    with path.open("r", encoding="utf-8") as f:
        for chunk in iter(lambda: f.read(chunk_size), ""):
            for ch in chunk:
                if pending_prefix_zero:
                    if ch in ("x", "X"):
                        pending_prefix_zero = False
                        index += 1
                        continue
                    push_hex("0", pending_prefix_pos)
                    pending_prefix_zero = False

                if ch.isspace():
                    index += 1
                    continue

                if not saw_digit and pending is None and ch == "0":
                    pending_prefix_zero = True
                    pending_prefix_pos = index
                    index += 1
                    continue

                push_hex(ch, index)
                index += 1

    if pending_prefix_zero:
        push_hex("0", pending_prefix_pos)

    if not saw_digit:
        raise ValueError("hex 输入为空")
    if pending is not None:
        raise ValueError("hex 输入长度必须为偶数")
    return bytes(output)


def _validate_input_args(encoded: str | None, file: Path | None) -> None:
    """校验输入参数互斥关系."""
    if encoded is None and file is None:
        raise ValueError("必须提供 ENCODED 参数或 --file 选项")
    if encoded is not None and file is not None:
        raise ValueError("不能同时使用 ENCODED 参数和 --file 选项")


def _read_input(
    encoded: str | None,
    file: Path | None,
    file_format: str,
) -> InputBuffer:
    """读取 CLI 输入并返回缓冲对象."""
    if file is None:
        assert encoded is not None
        return InputBuffer(data=parse_hex_string(encoded))

    if file_format == "hex":
        return InputBuffer(data=_parse_hex_stream(file))

    if file.stat().st_size == 0:
        return InputBuffer(data=b"")

    with file.open("rb") as f:
        mm = mmap.mmap(f.fileno(), length=0, access=mmap.ACCESS_READ)
    view = memoryview(mm)
    return InputBuffer(data=view, mm=mm, view=view)


def _decode_payload(data: bytes | memoryview, fmt: str) -> Any:
    """按输出模式执行解码."""
    if fmt == "tree":
        if isinstance(data, memoryview):
            return decode_trace(data.tobytes())
        return decode_trace(data)
    return decode_raw(data)


def _allow_probe(
    payload: bytes, depth: int, policy: ProbePolicy, rt: ProbeRuntime
) -> bool:
    """根据策略判断是否允许探测."""
    if policy.mode == "off":
        return False
    if rt.probed_nodes >= policy.max_nodes:
        return False
    if depth > policy.max_depth:
        return False
    if policy.mode == "auto" and len(payload) > policy.max_bytes:
        return False
    rt.probed_nodes += 1
    return True


def _probe_bytes(
    payload: bytes, depth: int, policy: ProbePolicy, rt: ProbeRuntime
) -> Any | None:
    """带缓存地探测 bytes 是否可解释为 Struct."""
    if payload in rt.probe_cache:
        return rt.probe_cache[payload]
    if not _allow_probe(payload, depth, policy, rt):
        rt.probe_cache[payload] = None
        return None
    result = probe_struct(payload)
    rt.probe_cache[payload] = result
    return result


def _decode_trace_cached(payload: bytes, rt: ProbeRuntime) -> TraceNode:
    """带缓存地执行 decode_trace."""
    if payload in rt.trace_cache:
        return rt.trace_cache[payload]
    trace = decode_trace(payload)
    rt.trace_cache[payload] = trace
    return trace


def deep_probe(data: Any, policy: ProbePolicy, rt: ProbeRuntime, depth: int = 0) -> Any:
    """递归探测并解码 bytes 中的 Struct."""
    if isinstance(data, dict):
        return {k: deep_probe(v, policy, rt, depth + 1) for k, v in data.items()}
    if isinstance(data, list):
        return [deep_probe(item, policy, rt, depth + 1) for item in data]
    if isinstance(data, bytes):
        struct = _probe_bytes(data, depth, policy, rt)
        if struct:
            return deep_probe(struct, policy, rt, depth + 1)
        return data
    return data


def build_trace_tree(
    node: TraceNode,
    policy: ProbePolicy,
    rt: ProbeRuntime,
    parent: TreeType | None = None,
    depth: int = 0,
) -> TreeType:
    """基于 TraceNode 构建 rich Tree."""
    from rich.text import Text
    from rich.tree import Tree

    label = Text()

    if node.jce_type != "ROOT":
        label.append(f"Tag {node.tag} ", style="bold blue")

    type_desc = node.jce_type
    if node.type_name:
        type_desc = f"{node.type_name}"
    label.append(f"({type_desc})", style="cyan")

    if node.name:
        label.append(f" [{node.name}]", style="yellow")

    if node.value is not None:
        val_str = str(node.value)
        if isinstance(node.value, str):
            val_str = repr(node.value)
        elif isinstance(node.value, bytes):
            preview = node.value[:16].hex().upper()
            if len(node.value) > 16:
                val_str = f"<{len(node.value)} bytes> {preview}..."
            else:
                val_str = f"<{len(node.value)} bytes> {preview}"
        label.append(": ", style="white")
        label.append(val_str, style="green")

    tree = Tree(label) if parent is None else parent.add(label)

    for child in node.children:
        build_trace_tree(child, policy, rt, tree, depth + 1)

    if node.jce_type == "SimpleList" and isinstance(node.value, bytes):
        struct = _probe_bytes(node.value, depth, policy, rt)
        if struct:
            try:
                inner_trace = _decode_trace_cached(node.value, rt)
            except Exception:
                return tree
            inner_branch = tree.add(
                Text(">>> Probed Structure >>>", style="bold magenta")
            )
            for child in inner_trace.children:
                build_trace_tree(child, policy, rt, inner_branch, depth + 1)

    return tree


class BytesEncoder(json.JSONEncoder):
    """自定义 JSON encoder, 处理 bytes."""

    def default(self, o: Any) -> Any:
        """将 bytes 转换为可序列化格式."""
        if isinstance(o, bytes):
            try:
                return o.decode("utf-8")
            except UnicodeDecodeError:
                return f"0x{o.hex()}"
        return super().default(o)


def _prepare_dict_data(data: Any, policy: ProbePolicy, rt: ProbeRuntime) -> Any:
    """将解码结果转为纯 dict 结构以供输出."""
    output_data = data
    if hasattr(data, "to_dict"):
        output_data = data.to_dict()

    if isinstance(output_data, dict):
        output_data = deep_probe(output_data, policy, rt)
    return output_data


def format_output(
    data: Any,
    fmt: str,
    console: ConsoleType,
    policy: ProbePolicy,
    rt: ProbeRuntime,
) -> None:
    """格式化输出数据."""
    if fmt == "json":
        output_data = _prepare_dict_data(data, policy, rt)
        json_str = json.dumps(
            output_data,
            cls=BytesEncoder,
            indent=2,
            ensure_ascii=False,
        )
        from rich.syntax import Syntax

        syntax = Syntax(
            json_str,
            "json",
            theme="ansi_dark",
            background_color="default",
            word_wrap=True,
        )
        console.print(syntax)
        return

    if fmt == "tree":
        if not isinstance(data, TraceNode):
            console.print("[red]Internal Error: Tree format requires TraceNode[/]")
            return
        tree = build_trace_tree(data, policy, rt)
        console.print(tree)
        return

    output_data = _prepare_dict_data(data, policy, rt)
    console.print(output_data)


def _render_or_dump(
    decoded: Any,
    fmt: str,
    output: Path | None,
    console: ConsoleType,
    error_console: ConsoleType,
    policy: ProbePolicy,
    rt: ProbeRuntime,
) -> None:
    """渲染终端输出或写入文件."""
    if output is None:
        format_output(decoded, fmt, console, policy, rt)
        return

    try:
        save_data = _prepare_dict_data(decoded, policy, rt)
        with output.open("w", encoding="utf-8") as f:
            json.dump(
                save_data,
                f,
                cls=BytesEncoder,
                indent=2,
                ensure_ascii=False,
            )
    except (OSError, TypeError) as e:
        error_console.print(f"[red]Error:[/] 输出失败: {e}")
        raise SystemExit(1) from e
    console.print(f"[green]输出已保存到:[/] {output}")


def _create_cli() -> Any:
    """创建 CLI 命令."""
    _check_cli_deps()

    from rich.console import Console

    console = Console()
    error_console = Console(stderr=True)

    @click.command()
    @click.argument("encoded", required=False)
    @click.option(
        "-f",
        "--file",
        type=click.Path(exists=True, path_type=Path),
        help="从文件读取输入数据",
    )
    @click.option(
        "--file-format",
        type=click.Choice(["bin", "hex"]),
        default="bin",
        show_default=True,
        help="文件输入格式: bin 表示原始二进制, hex 表示十六进制文本",
    )
    @click.option(
        "--probe",
        type=click.Choice(["off", "auto", "on"]),
        default="auto",
        show_default=True,
        help="嵌套 bytes 探测策略",
    )
    @click.option(
        "--probe-max-bytes",
        type=click.IntRange(min=1),
        default=65536,
        show_default=True,
        help="probe=auto 时单个 bytes 允许探测的最大长度",
    )
    @click.option(
        "--probe-max-depth",
        type=click.IntRange(min=0),
        default=3,
        show_default=True,
        help="最大探测递归深度",
    )
    @click.option(
        "--probe-max-nodes",
        type=click.IntRange(min=1),
        default=256,
        show_default=True,
        help="单次执行最多探测的 bytes 节点数量",
    )
    @click.option(
        "--format",
        "fmt",
        type=click.Choice(["pretty", "json", "tree"]),
        default="pretty",
        show_default=True,
        help="输出格式",
    )
    @click.option(
        "-o",
        "--output",
        type=click.Path(path_type=Path),
        help="将输出保存到文件",
    )
    @click.option(
        "-v",
        "--verbose",
        is_flag=True,
        help="显示详细的解码过程信息",
    )
    def cli(
        encoded: str | None,
        file: Path | None,
        file_format: str,
        probe: str,
        probe_max_bytes: int,
        probe_max_depth: int,
        probe_max_nodes: int,
        fmt: str,
        output: Path | None,
        verbose: bool,
    ) -> None:
        """Tars 编解码命令行工具.

        Examples:
            tarsio "00 64"
            tarsio -f payload.bin --format json
        """
        try:
            _validate_input_args(encoded, file)
        except ValueError as e:
            error_console.print(f"[red]Error:[/] {e}")
            raise SystemExit(1) from e

        policy = ProbePolicy(
            mode=probe,
            max_bytes=probe_max_bytes,
            max_depth=probe_max_depth,
            max_nodes=probe_max_nodes,
        )
        rt = ProbeRuntime()

        try:
            input_buffer = _read_input(encoded, file, file_format)
        except (OSError, UnicodeDecodeError, ValueError) as e:
            error_console.print(f"[red]Error:[/] 输入读取失败: {e}")
            raise SystemExit(1) from e

        try:
            if verbose:
                console.print(
                    f"[dim][INFO] 输入大小: {len(input_buffer.data)} bytes[/]"
                )
                preview_hex = input_buffer.data[:50].hex()
                formatted_hex = " ".join(
                    preview_hex[i : i + 2] for i in range(0, len(preview_hex), 2)
                )
                if len(input_buffer.data) > 50:
                    formatted_hex += " ..."
                console.print(f"[dim][DEBUG] Hex: {formatted_hex}[/]")
                if file is not None:
                    if file_format == "hex":
                        console.print("[dim][INFO] 文件输入格式: hex 文本[/]")
                    else:
                        console.print("[dim][INFO] 文件输入格式: 二进制[/]")

            try:
                decoded = _decode_payload(input_buffer.data, fmt)
            except Exception as e:
                error_console.print(f"[red]Error:[/] 解码失败: {e}")
                raise SystemExit(1) from e

            _render_or_dump(decoded, fmt, output, console, error_console, policy, rt)
        finally:
            input_buffer.close()

    return cli


def main() -> None:
    """入口函数."""
    cli = _create_cli()
    cli()


if __name__ == "__main__":
    main()



"""Tarsio Raw benchmark errors."""

import pytest
from tarsio._core import decode_raw


@pytest.mark.benchmark(group="raw_error")
def test_bench_raw_decode_truncated(benchmark, raw_truncated_bytes):
    """测试 Raw 模式解码截断数据."""

    def _decode():
        with pytest.raises(ValueError):  # noqa: PT011
            decode_raw(raw_truncated_bytes)

    benchmark(_decode)



"""Payloads benchmarks (Large data, Zero copy)."""

import pytest
from tarsio._core import decode, encode

from .models import LargeData


@pytest.mark.benchmark(group="large_data")
def test_bench_encode_large_data(benchmark, large_data_obj):
    """测试大数据块(Bytes vs List[int])编码性能."""
    benchmark(encode, large_data_obj)


@pytest.mark.benchmark(group="large_data")
def test_bench_decode_large_data(benchmark, large_data_bytes):
    """测试大数据块(Bytes vs List[int])解码性能."""
    benchmark(decode, LargeData, large_data_bytes)


@pytest.mark.benchmark(group="zero_copy")
def test_bench_decode_huge_blob(benchmark, huge_blob_bytes):
    """测试 10MB Blob 解码性能 (检测 Zero-copy)."""
    benchmark(decode, LargeData, huge_blob_bytes)
